/***************************************************************************
                          caddfiles.cpp  -  description
                             -------------------
    begin                : Fri Jul 21 2000
    copyright            : (C) 2000 by Eric Coquelle
    email                : coquelle@caramail.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include "caddfiles.h"

CAddFiles::CAddFiles(QWidget *parent, const char *name) : QDialog(parent,name,true){
	initDialog();
	
	connect(bBrowseAdd, SIGNAL(clicked()), this, SLOT(slotBrowseAddToOtherArchive()));
	connect(bBrowseCreate, SIGNAL(clicked()), this, SLOT(slotBrowseCreateArchive()));
	connect(bOk, SIGNAL(clicked()), this, SLOT(slotOk()));
	connect(bCancel, SIGNAL(clicked()), this, SLOT(slotCancel()));
}

CAddFiles::~CAddFiles(){
}

/** The user will browse and choose the archive
to which the files will be added */
void CAddFiles::slotBrowseAddToOtherArchive(){
	CChoixFichier archiveName;
	QString name;
	
	name=archiveName.openArchive();
	if(!name.isEmpty())
		leAddOther->setText(name);
}

/** The user will enter a new archive name */
void CAddFiles::slotBrowseCreateArchive(){
	QString filtre;
	QString name;
	
	filtre="*.tar.gz *.tgz *.Z|";
	filtre+=i18n("targz archives");	
	filtre+="\n*.tar.bz2|";
	filtre+=i18n("tarbz2 archives");	
	filtre+="\n*.lha *.lzh|";
	filtre+=i18n("lha archives");
	filtre+="\n*.arj|";
	filtre+=i18n("arj archives");
	filtre+="\n*.rar|";
	filtre+=i18n("rar archives");
	filtre+="\n*.gz|";
	filtre+=i18n("pure gzip archives");
	filtre+="\n*.bz2|";
	filtre+=i18n("pure bzip2 archives");
	filtre+="\n*.zip|";
	filtre+=i18n("zip archives");
	filtre+="\n*.tar|";
	filtre+=i18n("pure tar archives");	
	
	name=KFileDialog::getSaveFileName(QDir::homeDirPath(), filtre );
	
	if(!name.isEmpty())
		leCreate->setText(name);
}

/** Return 0 if we'll add files to current archive,
1 if we'll add to another,
2 if we'll create a new archive
3 if we'll directly open this archive*/
int CAddFiles::getAddFilesChoice(){
	if(rbAddCurrent->isChecked())
		return 0;
	else if(rbAddOther->isChecked())
		return 1;
	else if(rbCreate->isChecked())
		return 2;
	else if(rbOpenArchive->isChecked())
		return 3;
	return -1;
}

/** return the name of the archive to which
the files will be added */
QString CAddFiles::getOtherArchiveName(){
	return leAddOther->text();
}

/** return the name of the archive to be created */
QString CAddFiles::getNewArchiveName(){
	return leCreate->text();
}

/** Validate the choice */
void CAddFiles::slotOk(){
	accept();
}

/** reject the choices */
void CAddFiles::slotCancel(){
	reject();
}

/** The dropped file is an archive, so modify
the dialog box. */
void CAddFiles::setCheckBoxArchive(){
	rbCreate->hide();
	bBrowseCreate->hide();
	leCreate->hide();
	rbOpenArchive->show();
	rbOpenArchive->setChecked(true);
}

/** If karchiveur doesn't contain any archive,
	* disable the first radio button */
void CAddFiles::canAddToCurrentArchive(bool canAdd){
	if(!canAdd)
	{
		rbAddCurrent->setEnabled(false);
		rbCreate->setChecked(true);
	}
}
#include "caddfiles.moc"
