/***************************************************************************
                          carchive.cpp  -  description
                             -------------------
    begin                : Sat Dec 2 2000
    copyright            : (C) 2000 by Eric Coquelle
    email                : coquelle@caramail.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "carchive.h"

#include <stdio.h>

#include <kiconloader.h>

QListView* CArchive::list=NULL;
QString CArchive::archiveName="";
CLed* CArchive::led=NULL;
KStatusBar* CArchive::statusbar=NULL;
int CArchive::iconsize=16;
int CArchive::kindofdate=STANDARD_DATE;
KProgress* CArchive::progressbar=NULL;
bool CArchive::overwrite=true;
bool CArchive::recursivemode=true;
bool CArchive::displayicons=true;
int CArchive::compressrate=9;

CArchive::CArchive(){
	readArchiveWithStream=true;
	stopreadprocess=false;
}

CArchive::~CArchive(){
}

/** display a message in the statusbar */
void CArchive::displayMessage( QString message ){
	if(statusbar==NULL)
		return;
  statusbar->clear();
  statusbar->changeItem(message, ID_STATUS_MSG);
}

/** Print in the statusbar the size of the loaded
	archive */
void CArchive::displayMessageArchiveSize(){
  QString message;
  
  message=QString("%1  -  %2 %3").arg(IDS_STATUS_DEFAULT).arg(archiveSize()).arg(i18n("bytes"));
  displayMessage(message);
}

/** return the size of current archive */
int CArchive::archiveSize(){
  FILE *F1;
  long tailleArchive;
  
  tailleArchive=0;
  if ((F1 = fopen(/*(const char*)*/archiveName, "rb")) == NULL)
    return 0;
  fseek(F1, 0, SEEK_END);
  tailleArchive=ftell(F1);
  fclose(F1);
  return tailleArchive;
}

/** Find which icon to insert in the listview, by
examinating the extension of files the archive contains */
void CArchive::definitIcone(QString name, QString permissions, CListViewItem* elementListe){
	if(!displayicons)
		return;

		KIconLoader icone;
		QString ext2;
		QString ext3;
		QString ext4;
		QString ext5;
		QString ext6;
		QString ext8;

		name=name.lower();

		ext2=name.right(2);
		ext3=name.right(3);
		ext4=name.right(4);
		ext5=name.right(5);
		ext6=name.right(6);
		ext8=name.right(8);

		if(name=="..")
			elementListe->setPixmap(0,icone.loadIcon("folder.png",KIcon::Small, iconsize));
		else if(permissions.contains('x',FALSE)!=0)
			elementListe->setPixmap(0,icone.loadIcon("exec.png",KIcon::Small, iconsize));
		else if(ext2==".h")
			elementListe->setPixmap(0,icone.loadIcon("karchiveur_h.png",KIcon::Small, iconsize));
		else if((ext4==".cpp")||(ext3==".cc")||(ext2==".c"))
			elementListe->setPixmap(0,icone.loadIcon("karchiveur_cpp.png",KIcon::Small, iconsize));
		else if(ext4==".txt")
			elementListe->setPixmap(0,icone.loadIcon("txt.png",KIcon::Small, iconsize));
		else if(name=="makefile")
			elementListe->setPixmap(0,icone.loadIcon("make",KIcon::Desktop, iconsize));
		else if((name=="readme")||(name=="install")||(name=="authors")||(name=="copying")||(name=="todo"))
			elementListe->setPixmap(0,icone.loadIcon("karchiveur_gnu.png",KIcon::Small, iconsize));
		else if((name=="changelog")||(ext4==".lsm"))
			elementListe->setPixmap(0,icone.loadIcon("info.png",KIcon::Small, iconsize));
		else if(ext3==".ps")
			elementListe->setPixmap(0,icone.loadIcon("postscript.png",KIcon::Small, iconsize));
		else if(ext4==".rpm")
			elementListe->setPixmap(0,icone.loadIcon("rpm.png",KIcon::Small, iconsize));
		else if((ext4==".gif")||(ext4==".jpg")||(ext4==".bmp")||(ext4==".png")||(ext4==".xpm"))
			elementListe->setPixmap(0,icone.loadIcon("image.png",KIcon::Small, iconsize));
		else if(name=="configure")
			elementListe->setPixmap(0,icone.loadIcon("exec.png",KIcon::Small, iconsize));
		else if((ext4==".htm")||(ext5==".html")||(ext6==".shtml"))
			elementListe->setPixmap(0,icone.loadIcon("html.png",KIcon::Small, iconsize));
		else if((ext4==".wav")||(ext3==".au")||(ext3==".xm")||(ext4==".s3m")||(ext4==".mod")||(ext4==".mp3"))
			elementListe->setPixmap(0,icone.loadIcon("karchiveur_son.png",KIcon::Small, iconsize));
		else if((ext3==".po")||(ext4==".gmo")||(ext4==".pot"))
			elementListe->setPixmap(0,icone.loadIcon("locale.png",KIcon::Small, iconsize));
		else if((ext3==".in")||(ext3==".am"))
			elementListe->setPixmap(0,icone.loadIcon("karchiveur_in.png",KIcon::Small, iconsize));
		else if(name.right(1)=="~")
			elementListe->setPixmap(0,icone.loadIcon("karchiveur_trash.png",KIcon::Small, iconsize));
		else if((ext3==".gz")||(ext4==".bz2")||(ext4==".tar")||(ext4==".zip")||(ext2==".z"))
			elementListe->setPixmap(0,icone.loadIcon("karchiver.png",KIcon::Small, iconsize));
		else if(ext3==".m4")
			elementListe->setPixmap(0,icone.loadIcon("karchiveur_m4.png",KIcon::Small, iconsize));
		else if((ext5==".diff")||(name.left(5)=="patch")||(ext5=="patch"))
			elementListe->setPixmap(0,icone.loadIcon("karchiveur_diff.png",KIcon::Small, iconsize));
		else if((ext6==".class")||(ext5==".java")||(ext4==".jar"))
			elementListe->setPixmap(0,icone.loadIcon("karchiveur_java.png",KIcon::Small, iconsize));
		else if((ext8==".kdevprj")||(name.right(7)==".kdelnk")||(ext8==".kdevdlg"))
			elementListe->setPixmap(0,icone.loadIcon("karchiveur_mak.png",KIcon::Small, iconsize));
}

/** prepare the local variable, clear the listview
before reading any archive */
void CArchive::initializeReadingArchive(){
	stopreadprocess=false;
	counter=0;
  displayMessage(i18n("Running compressor..."));
  processread.clearArguments();
}

/** set the main widget so as to this class can
draw the archive content */
void CArchive::setWidgetListView( QListView *l ){
	list=l;
}

/** set an access to the led widget so as this
class can draw it */
void CArchive::setWidgetLed( CLed *l ){
	led=l;
}

/** set an access to the statusbar, so as this
class can draw in it */
void CArchive::setStatusBar( KStatusBar *b ){
	statusbar=b;
}

/** If @param read is true, read current archive
with a stream, ie by a quick way, but this
freezes karchiveur's interface */
void CArchive::setReadArchiveWithStream( bool read ){
	readArchiveWithStream=read;
}

void CArchive::displayArchiveContent() {
}

void CArchive::extractArchive(QString & , int , QString & ) {
}

void CArchive::removeFilesFromArchive(QStrList ) {
}

void CArchive::addFilesToArchive( QStrList , bool, int , const char* ) {
}

void CArchive::createArchive(QString , QStrList, const char* ) {
}

/** set the name of the working archive to
@param archName */
void CArchive::setArchiveName( QString archName ){
	archiveName=archName;
}

/** the the format of displayed date to d
 */
void CArchive::setKindOfDate( int d ){
	kindofdate=d;
}

/** if @param overwr is true, one will overwrite
files when extracting files */
void CArchive::setOverwriteFilesWhenExtracting( bool overwr ){
	overwrite=overwr;
}

/** A link to the progressbar */
void CArchive::setWidgetProgressBar( KProgress *progress ){
	progressbar=progress;
}

/** Kill the read process */
void CArchive::stopReading(){
	stopreadprocess=true;
	processread.kill();

}
/** Kill the extract process */
void CArchive::stopExtracting(){
	processextract.kill();
}

/** Set the size of the icons in the listview to @param size */
void CArchive::setIconSize( int size ){
	iconsize=size;
}

/** If true, when adding a directory, also add the subdirs */
void CArchive::setRecursiveMode (bool rec){
	recursivemode=rec;
}

/** if @param icons is false, don't display the
icons, to speed up archive reading */
void CArchive::setDisplayIcons( bool icons ){
	displayicons=icons;
}

/** If avaible for the compressor, set the compress
rate to @param rate. @param rate must be in [1,9] */
void CArchive::setCompressRate( int rate ){
	compressrate=rate;
}
#include "carchive.moc"
