/***************************************************************************
                          cchoixfichier.cpp  -  description                              
                             -------------------                                         
    begin                : Thu Jun 3 1999                                           
    copyright            : (C) 1999 by Eric Coquelle                         
    email                : coquelle@caramail.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/

#include <kstddirs.h>

#include "cchoixfichier.h"


CChoixFichier::CChoixFichier(QWidget *parent, const char *name )
	: QWidget( parent, name )
{
	m_path=QDir::home();
	m_previouspath=QDir::home();
	typeDernierRepertoire=1;
	filtre="*.lha *.lzh *.arj *.rar *.gz *.bz2 *.zip *.tar *.tgz *.Z|";
	filtre+=i18n("All valid archives");
	filtre+="\n*.lha *.lzh|";
	filtre+=i18n("lha archives");
	filtre+="\n*.arj|";
	filtre+=i18n("arj archives");
	filtre+="\n*.rar|";
	filtre+=i18n("rar archives");
	filtre+="\n*.gz|";
	filtre+=i18n("gzip archives");
	filtre+="\n*.bz2|";
	filtre+=i18n("bzip2 archives");
	filtre+="\n*.zip|";
	filtre+=i18n("zip archives");
	filtre+="\n*.tar *.tar.gz *.tar.bz2 *.tgz *.Z|";
	filtre+=i18n("tar archives");	
	boitedlg=new KFileDialog(m_path.path(), filtre, this,"", false);	
}

CChoixFichier::~CChoixFichier(){
	delete boitedlg;
}

void CChoixFichier::initialise()
{
	decompresseur="";
	listeur="";
	fichierchoisit="";
	typeCompresseur=-1; //No valid archive loaded
}

QString CChoixFichier::openArchive()
{
	fichierchoisit=boitedlg->getOpenFileName(m_path.path(),filtre);
	return fichierchoisit;
}

int CChoixFichier::findKindOfCompressor()
{
	QString archive;
	
	if(fichierchoisit.isNull())
		return 2;

	archive=fichierchoisit.lower();

	progrCompresseur="tar";
	typeCompresseur=UNKNOWN;
	optionSupp="";
	compresserconcatener="";
	compressoravaible=false;

	if((archive.right(7)==".tar.gz")||(archive.right(6)==".tar.z")||(archive.right(4)==".tgz")||(archive.right(7)=="_tar.gz")||(archive.right(6)=="_tar.z")||(archive.right(4)=="_tgz"))
	{
			decompresseur="-zvxf";
			listeur="-zvtf";
			compresserconcatener="-r";
			typeCompresseur=TARGZ;
			if( (KStandardDirs::findExe("gzip")!=QString::null)&&(KStandardDirs::findExe(progrCompresseur)!=QString::null))
				compressoravaible=true;
	}
	else if((archive.right(8)==".tar.bz2")||(archive.right(8)=="_tar.bz2"))
	{
			optionSupp="--use-compress-program=bzip2";
			decompresseur="-vxf";
			listeur="-vtf";
			typeCompresseur=TARBZ2;
			if( (KStandardDirs::findExe("bzip2")!=QString::null)&&(KStandardDirs::findExe(progrCompresseur)!=QString::null))
				compressoravaible=true;
	}
	else if(archive.right(4)==".zip")
	{
			progrCompresseur="unzip";
			listeur="-lv";
			decompresseur="";
			typeCompresseur=ZIP;
		if(KStandardDirs::findExe(progrCompresseur)!=QString::null)
			compressoravaible=true;
	}
	else if((archive.right(4)==".tar")||(archive.right(4)=="_tar"))
	{
		listeur="-vtf";
		decompresseur="-vxf";
		typeCompresseur=TAR;
		if(KStandardDirs::findExe(progrCompresseur)!=QString::null)
			compressoravaible=true;
	}
	else if(archive.right(3)==".gz")
	{
		progrCompresseur="gzip";
		listeur="-lv";
		decompresseur="-dv";
		typeCompresseur=GZIP;
		if(KStandardDirs::findExe(progrCompresseur)!=QString::null)
			compressoravaible=true;
	}
	else if(archive.right(4)==".bz2")
	{
		progrCompresseur="bzip2";
		listeur="";
		decompresseur="-dvk";
		typeCompresseur=BZIP2;
		if(KStandardDirs::findExe(progrCompresseur)!=QString::null)
			compressoravaible=true;
	}
	else if(archive.right(4)==".rar")
	{
		progrCompresseur="unrar";
		listeur="l";
		decompresseur="x";
		typeCompresseur=RAR;
		if(KStandardDirs::findExe(progrCompresseur)!=QString::null)
			compressoravaible=true;
	}
	else if((archive.right(4)==".lzh")||(archive.right(4)==".lha"))
	{
		progrCompresseur="lha";
		listeur="l";
		decompresseur="x";
		typeCompresseur=LHA;
		if(KStandardDirs::findExe(progrCompresseur)!=QString::null)
			compressoravaible=true;
	}	
	else if(archive.right(4)==".arj")
	{
		progrCompresseur="unarj";
		listeur="l";
		decompresseur="e";
		typeCompresseur=ARJ;
		if(KStandardDirs::findExe(progrCompresseur)!=QString::null)
			compressoravaible=true;
	}
	else
		return 1;
		
  //As the archive is valid, we set the last open directory
  m_previouspath=fichierchoisit.left(fichierchoisit.findRev('/'));
  if(typeDernierRepertoire==0)
  	m_path=m_previouspath;
	return 0;
}

QString CChoixFichier::getCompresseur()
{
return compresseur;
}

QString CChoixFichier::getDecompresseur()
{
	return decompresseur;
}

QString CChoixFichier::getListeur()
{
	return listeur;
}

QString CChoixFichier::getNomFichier()
{
	return fichierchoisit;
}

QString CChoixFichier::getProgrCompresseur()
{
	return progrCompresseur;
}

QString CChoixFichier::getOptionSupp()
{
	return optionSupp;
}

QString CChoixFichier::getConcateneur()
{
	return compresserconcatener;
}

void CChoixFichier::setNomArchive(QString archive)
{
  fichierchoisit=archive;
}

/** Set the acess path when opening an archive */
void CChoixFichier::setCheminAcces(QString  chemin){
	m_path=chemin;
	if(!m_path.isReadable())
		m_path=QDir::home();
}

/** return the directory's path of the opened file */
QString CChoixFichier::getCheminAcces(){
	return m_path.path();
}

/** Define the default path when opening a file
0: Last dir
1: home dir
2: shell's one */
void CChoixFichier::setCheminAcces(QString chemin, int optionOuverture){
	typeDernierRepertoire=optionOuverture;
	m_path=chemin;
	
	switch(optionOuverture)
	{
		case 0:
			setCheminAcces(chemin);
			break;
		case 1:
			m_path=QDir::home();
			break;
		case 2:
			m_path.setPath("");
			break;
	}
}

/** returns the exact compressor type */
int CChoixFichier::getKindOfCompressor(){
	return typeCompresseur;
}

/** Define the default path when opening a file
0: Last dir
1: home dir
2: shell's one */
void CChoixFichier::setOptionOuverture(int optionOuverture){
	typeDernierRepertoire=optionOuverture;
	switch(optionOuverture)
	{
		case 0:
			m_path=m_previouspath;
		case 1:
			m_path=QDir::home();
			break;
		case 2:
			m_path.setPath("");
			break;
	}
}

/** return the directory of the current archive */
QString CChoixFichier::getCurrentArchiveDirectory(){
  return m_previouspath.path();
}

/** We store somewhere an oldpath variable,
    used to know what is the last open dir */
void CChoixFichier::setForInitPreviousPath( QString path ){
  m_previouspath=path;
  if(!m_previouspath.isReadable())
    m_previouspath=QDir::home();
}

/** Return true if the compressor that'll be used is installed on disk */
bool CChoixFichier::isCurrentCompressorAvaible(){
	return compressoravaible;
}

#include "cchoixfichier.moc"
