/***************************************************************************
                          cpreferences.cpp  -  description
                             -------------------
    begin                : Sat Jul 1 2000
    copyright            : (C) 2000 by Eric Coquelle
    email                : coquelle@caramail.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "cpreferences.h"

CPreferences::CPreferences(QWidget *parent, const char *name) :KDialogBase( IconList, i18n("Configure"), Help|Default|User1|Ok|Cancel,
		Ok, parent, name, true, true, i18n("Save") ) {

  setHelp( "karchiver/index.html", QString::null );

	setGzipOptions();
	setTarOptions();
	setDirectoriesOptions();
	setIconSizeOptions();
	setArchiveReadingOptions();
	setCompresseursOptions();
	setDateOptions();
	setFontOptions();
	setTipofDayOptions();
	setMiscOptions();	
	
	//Aditionnal init
	setCaption(i18n("Configuration"));
	
	connect(this, SIGNAL(okClicked()), this, SLOT(slotOk()));
	connect(this, SIGNAL(user1Clicked()), this, SLOT(slotSauver()));
	connect(this, SIGNAL(cancelClicked()), this, SLOT(slotAnnuler()));
	connect(this, SIGNAL(defaultClicked()), this, SLOT(slotDefault()));
	
	connect(cbOGZEcrase, SIGNAL(clicked()), cbOTEcrase, SLOT(toggle()));
	connect(cbOTEcrase, SIGNAL(clicked()), cbOGZEcrase, SLOT(toggle()));
	
	litOptions();
	compresseursDispo();	
}

CPreferences::~CPreferences(){
}

/*Read the config file and set the propper values for all widgets*/
void CPreferences::litOptions()
{
  KConfig *config = kapp->config();
  QFont font;

  config->setGroup("OPTIONS");
	cbOTCheminAbsolu->setChecked(config->readBoolEntry("CheminAbsolu",false));
  cbOTToutChemin->setChecked(config->readBoolEntry("ToutChemin",false));
  cbOTRecursif->setChecked(config->readBoolEntry("Recursif",true));
	cbOGZEcrase->setChecked(config->readBoolEntry("EcraserGzip",true));
	cbOTEcrase->setChecked(config->readBoolEntry("EcraserGzip",true));
	cmbOGZTauxCompression->setCurrentItem(config->readNumEntry("TauxCompression",9)-1);

	
	switch(config->readNumEntry("LectureParFlot",1))
	{
		case 0:
			rbOLRapide->setChecked(true);
			break;
		case 1:
			rbOLLent->setChecked(true);
			break;
		case 2:
			rbOLVeryFast->setChecked(true);
			break;
	}
	
	switch(config->readNumEntry("SelectionMode",0))
	{
		case 0:
			rbMOSMMulti->setChecked(true);
			break;
		case 1:
			rbMOSMExtended->setChecked(true);
			break;
	}
		
	config->setGroup("LASTDIRS");

	switch(config->readNumEntry("OptionExtraction",1))
	{
		case 0:
			rbDDEDDernier->setChecked(true);
			break;
		case 1:
			rbDDEDMaison->setChecked(true);
			break;
		case 2:
			rbDDEDShell->setChecked(true);
			break;
	}
	switch(config->readNumEntry("OptionOuverture",1))
	{
		case 0:
			rbDDODDernier->setChecked(true);
			break;
		case 1:
			rbDDODMaison->setChecked(true);
			break;
		case 2:
			rbDDODShell->setChecked(true);
			break;
	}
	
	config->setGroup("ICONES");

	switch(config->readNumEntry("TailleIconesListe",0))
	{
		case 0:
			rbISFPetit->setChecked(true);
			break;
		case 1:
			rbISFMoyen->setChecked(true);
			break;
		case 2:
			rbISFGrand->setChecked(true);
			break;
	}
		
	config->setGroup("FONT");
	
	font.setFamily(config->readEntry("Family","Times"));
	font.setPointSize(config->readNumEntry("Size",10));
	font.setWeight(config->readNumEntry("Weight",50));	
	font.setItalic(config->readBoolEntry("Italic",false));
	mFont->setFont(font);
	
	config->setGroup("DATE");

	switch(config->readNumEntry("KindOfDate",0))
	{
		case 0:
			rbDDStandard->setChecked(true);
			break;
		case 1:
			rbDDLocalisedSmall->setChecked(true);
			break;
		case 2:
			rbDDLocalisedLarge->setChecked(true);
			break;
	}
	
	config->setGroup("TipOfTheDay");
	cbOTDDisplayTipofDay->setChecked(config->readBoolEntry("show_tod",true));
	
}

/*Save those options*/
void CPreferences::Sauver()
{
  KConfig *config = kapp->config();


  config->setGroup("OPTIONS");
	config->writeEntry("CheminAbsolu",getCheminAbsolu());
  config->writeEntry("ToutChemin",getToutChemin());
  config->writeEntry("Recursif",getRecursif());
	config->writeEntry("EcraserGzip",getEcraserGzip());
	config->writeEntry("EcraserTar",getEcraserTar());
	config->writeEntry("TauxCompression",getTauxCompression());
	config->writeEntry("LectureParFlot",getLitParFlot());
	config->writeEntry("SelectionMode",getSelectionMode());
	
	config->setGroup("LASTDIRS");
	config->writeEntry("OptionExtraction",getTypeRepertoireExtraction());
	config->writeEntry("OptionOuverture",getTypeRepertoireOuverture());

	config->setGroup("ICONES");
	config->writeEntry("TailleIconesListe",getTailleIconeListe());
	
	config->setGroup("FONT");
	config->writeEntry("Family",mFont->font().family());
	config->writeEntry("Size",mFont->font().pointSize());
	config->writeEntry("Weight",mFont->font().weight());	
	config->writeEntry("Italic",mFont->font().italic());
	
	config->setGroup("DATE");
	config->writeEntry("KindOfDate",getKindOfDate());
	
	config->setGroup("TipOfTheDay");
	config->writeEntry("show_tod",cbOTDDisplayTipofDay->isChecked());
}


/** returns true if we will use an absolute path
 * for files added to the archive */
bool CPreferences::getCheminAbsolu(){
	return cbOTCheminAbsolu->isChecked();
}

/** return true if we will include the absoute path
	* and not the relative when dropping */
bool CPreferences::getToutChemin(){
	return cbOTToutChemin->isChecked();
}

/** return true if we will recurse folders when dropping */
bool CPreferences::getRecursif(){
	return cbOTRecursif->isChecked();
}

/** return the compress rate for gzip ans bzip2
1 = min, 9 = max */
int CPreferences::getTauxCompression(){
	return cmbOGZTauxCompression->currentItem()+1;
}

/** return true if we will overwrite files while
uncompressing gzip and bzip2 archives */
bool CPreferences::getEcraserGzip(){
	return cbOGZEcrase->isChecked();
}

/** return 0 if we'll open the last dir, 1 for home dir,
and 2 for shell's one */
int CPreferences::getTypeRepertoireExtraction(){
	if(rbDDEDDernier->isChecked())
		return 0;
	else if(rbDDEDMaison->isChecked())
		return 1;
	else if(rbDDEDShell->isChecked())
		return 2;
	return 1;
}

/** return 0 for last dir, 1 for home dir, 2 for shell's one
==>set the default directory when OPENING files */
int CPreferences::getTypeRepertoireOuverture(){
	if(rbDDODDernier->isChecked())
		return 0;
	else if(rbDDODMaison->isChecked())
		return 1;
	else if(rbDDODShell->isChecked())
		return 2;
	return 1;
}

/** return 0 if we'll display small icons, 1 for medium,
2 for large */
int CPreferences::getTailleIconeListe(){
	if(rbISFPetit->isChecked())
		return 0;
	else if(rbISFMoyen->isChecked())
		return 1;
	else if(rbISFGrand->isChecked())
		return 2;
	return 1;
}

/** close the dialog box and use the changes */
void CPreferences::slotOk(){
	accept();
}

/** Save the changes */
void CPreferences::slotSauver(){
	Sauver();
}

/** close the dialog box, and forget the changes */
void CPreferences::slotAnnuler(){
	reject();
}

/** return true if we'll read archive using streams,
false if we'll read using signal/slot mechanism */
int CPreferences::getLitParFlot(){
	if(rbOLRapide->isChecked())
		return 0;
	else if(rbOLLent->isChecked())
		return 1;
	else
		return 2;
}

/** verify the avaible compressors */
void CPreferences::compresseursDispo(){
	if(KStandardDirs::findExe("tar")==NULL)
		lTarDispo->setText(i18n("is NOT avaible"));
	if(KStandardDirs::findExe("gzip")==NULL)
		lGzipDispo->setText(i18n("is NOT avaible"));
	if(KStandardDirs::findExe("bzip2")==NULL)
		lBzip2Dispo->setText(i18n("is NOT avaible"));
	if(KStandardDirs::findExe("lha")==NULL)
		lLhaDispo->setText(i18n("is NOT avaible"));
	if(KStandardDirs::findExe("unrar")==NULL)
		lRarDispo->setText(i18n("is NOT avaible"));
	if(KStandardDirs::findExe("unzip")==NULL)
		lUnzipDispo->setText(i18n("is NOT avaible"));
	if(KStandardDirs::findExe("zip")==NULL)
		lZipDispo->setText(i18n("is NOT avaible"));
	if(KStandardDirs::findExe("unarj")==NULL)
		lUnarjDispo->setText(i18n("is NOT avaible"));
}

/** return true if files will be overwritten when
they are extracted */
bool CPreferences::getEcraserTar(){
	return /*cbOTEcrase*/cbOGZEcrase->isChecked();
}

/** set all the widgets for gzip options */
void CPreferences::setGzipOptions(){
	QLabel *label;
  QString text;

  QFrame *page = addPage( i18n("Gzip"), i18n("Gzip and Bzip2 options"),
			  BarIcon("tgz", KIcon::SizeMedium ) );
  QVBoxLayout *topLayout = new QVBoxLayout( page, 0, spacingHint() );

  QGridLayout *gbox = new QGridLayout( 5, 2 );
  topLayout->addLayout( gbox );


  text = i18n("Compress rate");
  label = new QLabel( text, page, "compress rate label" );
  gbox->addWidget( label, 0, 0 );
  text = i18n("9 = max");
  label = new QLabel( text, page, "max compress rate label" );
  gbox->addWidget( label, 1, 0 );
  text = i18n("1 = min");
  label = new QLabel( text, page, "min compress rate label" );
  gbox->addWidget( label, 2, 0 );
  cmbOGZTauxCompression = new QComboBox( false, page );
  cmbOGZTauxCompression->insertItem(i18n("1"));
  cmbOGZTauxCompression->insertItem(i18n("2"));
  cmbOGZTauxCompression->insertItem(i18n("3"));
  cmbOGZTauxCompression->insertItem(i18n("4"));
  cmbOGZTauxCompression->insertItem(i18n("5"));
  cmbOGZTauxCompression->insertItem(i18n("6"));
  cmbOGZTauxCompression->insertItem(i18n("7"));
  cmbOGZTauxCompression->insertItem(i18n("8"));
  cmbOGZTauxCompression->insertItem(i18n("9"));
  cmbOGZTauxCompression->setCurrentItem(8);
  gbox->addWidget(cmbOGZTauxCompression , 2, 1 );
  cbOGZEcrase= new QCheckBox(page,"cb ecrase fichiers");
  cbOGZEcrase->setText(i18n("Overwrite files when extracting"));
  cbOGZEcrase->setChecked(true);
  gbox->addWidget(cbOGZEcrase , 4, 0 );

  topLayout->addStretch(10);
}

/** set the widgets for the TAR tab */
void CPreferences::setTarOptions(){
  QFrame *page = addPage( i18n("Tar"), i18n("Tar options"),
			  BarIcon("tgz", KIcon::SizeMedium ) );
  QVBoxLayout *topLayout = new QVBoxLayout( page, 0, spacingHint() );

  QGridLayout *gbox = new QGridLayout( 3, 0 );
  topLayout->addLayout( gbox );

  cbOTCheminAbsolu= new QCheckBox(page,"NoName");
  QToolTip::add(cbOTCheminAbsolu,i18n("Store files as /home/toto/files\ninstead of home/toto/files"));
  cbOTCheminAbsolu->setText(i18n("Ablolute path: add / to the paths of files"));
  gbox->addWidget(cbOTCheminAbsolu , 0, 0 );

  cbOTToutChemin= new QCheckBox(page,"NoName");
  QToolTip::add(cbOTToutChemin,i18n("If checked, the base path of the dropped dir will be\n home/toto/dropped_dir instead of drop_dir"));
  cbOTToutChemin->setText(i18n("Add the base path when dropping"));
  gbox->addWidget(cbOTToutChemin , 1, 0 );

  cbOTRecursif= new QCheckBox(page,"NoName");
  cbOTRecursif->setText(i18n("Recurse folders when dropping"));
  gbox->addWidget(cbOTRecursif , 2, 0 );

  cbOTEcrase= new QCheckBox(page,"NoName");
  cbOTEcrase->setText(i18n("Overwrite files when extracting"));
  //WARNING:cbOGZEcrase is initialised in setGzipOptions() !
  gbox->addWidget(cbOTEcrase , 3, 0 );

  topLayout->addStretch(10);
}

/** set the widgets for default directories */
void CPreferences::setDirectoriesOptions(){
  QFrame *page = addPage( i18n("Directories"), i18n("Default directories"),
			  BarIcon("folder", KIcon::SizeMedium ) );

  QGridLayout *gbox = new QGridLayout( page, 4, 4, 5 );

  gbDDExtraction= new QButtonGroup(page,"to extract");
  gbDDExtraction->setTitle(i18n("For extracting"));
  gbox->addMultiCellWidget(gbDDExtraction,0, 4, 0, 1);

  QGridLayout *grid1 = new QGridLayout( gbDDExtraction, 3, 1, 15,1 );

  rbDDEDDernier= new QRadioButton(gbDDExtraction,"NoName");
  rbDDEDDernier->setText(i18n("Last dir"));
  grid1->addWidget( rbDDEDDernier, 0, 0);

  rbDDEDMaison= new QRadioButton(gbDDExtraction,"NoName");
  rbDDEDMaison->setText(i18n("Home dir (~/)"));
  grid1->addWidget( rbDDEDMaison, 1, 0);

  rbDDEDShell= new QRadioButton(gbDDExtraction,"NoName");
  rbDDEDShell->setText(i18n("Shell's one"));
  grid1->addWidget( rbDDEDShell, 2, 0);

  gbDDOuverture= new QButtonGroup(page,"to open");
  gbDDOuverture->setTitle(i18n("For opening"));
  gbox->addMultiCellWidget(gbDDOuverture, 0, 4, 2, 3);

  grid1 = new QGridLayout( gbDDOuverture, 3, 1, 15,1 );

  rbDDODDernier= new QRadioButton(gbDDOuverture,"NoName");
  rbDDODDernier->setText(i18n("Last dir"));
  grid1->addWidget( rbDDODDernier, 0, 0);

  rbDDODMaison= new QRadioButton(gbDDOuverture,"NoName");
  rbDDODMaison->setText(i18n("Home dir (~/)"));
  grid1->addWidget( rbDDODMaison, 1, 0);

  rbDDODShell= new QRadioButton(gbDDOuverture,"NoName");
  rbDDODShell->setText(i18n("Shell's one"));
  grid1->addWidget(rbDDODShell , 2, 0);

}

/** set the widgets for Icon size options tab */
void CPreferences::setIconSizeOptions(){
  QFrame *page = addPage( i18n("Icons"), i18n("Icon size"),
			  BarIcon("image", KIcon::SizeMedium ) );

  QGridLayout *gbox = new QGridLayout( page, 6, 4, 5 );

  gbISFichiers= new QButtonGroup(page,"NoName");
  gbISFichiers->setTitle(i18n("of files in archive"));
  gbox->addMultiCellWidget(gbISFichiers, 0, 6, 0, 1);
  
  QGridLayout *grid1 = new QGridLayout(gbISFichiers , 3, 1, 15,7 );

  
  rbISFPetit= new QRadioButton(gbISFichiers,"NoName");
  rbISFPetit->setText(i18n("Small"));
  grid1->addWidget(rbISFPetit, 0, 0);

  rbISFMoyen= new QRadioButton(gbISFichiers,"NoName");
  rbISFMoyen->setText(i18n("Medium"));
  grid1->addWidget(rbISFMoyen, 1, 0);

  rbISFGrand= new QRadioButton(gbISFichiers,"NoName");
  rbISFGrand->setText(i18n("Large"));
  grid1->addWidget(rbISFGrand, 2, 0);

}

/** set the widgets for reading archive options */
void CPreferences::setArchiveReadingOptions(){
  QFrame *page = addPage( i18n("Reading"), i18n("Method for reading archive"),
			  BarIcon("document", KIcon::SizeMedium ) );

  QGridLayout *gbox = new QGridLayout( page, 4, 1, 5 );

  bgLectureArchive= new QButtonGroup(page, "Lecture archive");
  bgLectureArchive->setTitle(i18n("Read:"));
  gbox->addMultiCellWidget(bgLectureArchive, 0, 4, 0, 1);

  QGridLayout *grid1 = new QGridLayout( bgLectureArchive, 3, 1, 15,7 );

  rbOLVeryFast= new QRadioButton(bgLectureArchive,"NoName");
  rbOLVeryFast->setText(i18n("Fastest but don't use icons"));
  grid1->addWidget(rbOLVeryFast, 0, 0);

  rbOLRapide= new QRadioButton(bgLectureArchive,"NoName");
  rbOLRapide->setText(i18n("Faster but freeze kar's interface"));
  grid1->addWidget(rbOLRapide, 1, 0);

  rbOLLent= new QRadioButton(bgLectureArchive,"NoName");
  rbOLLent->setText(i18n("Normaly, but the reading can be interupted"));
  grid1->addWidget(rbOLLent, 2, 0);

}

/** set the widgets for avaible compressors */
void CPreferences::setCompresseursOptions(){
  QFrame *page = addPage( i18n("Compressors"), i18n("Avaible compressors"),
			  BarIcon("info", KIcon::SizeMedium ) );

  QGridLayout *gbox = new QGridLayout( page, 8, 2, 3 );

  lZipDispo= new QLabel(page,"NoName");
  lZipDispo->setText(i18n("is avaible"));
  gbox->addWidget(lZipDispo, 6, 1);

  QLabel_17= new QLabel(page,"NoName");
  QLabel_17->setText(i18n("The Zip compressor..."));
  gbox->addWidget(QLabel_17, 6, 0);

  QLabel_5= new QLabel(page,"NoName");
  QLabel_5->setText(i18n("The TAR compressor..."));
  gbox->addWidget(QLabel_5, 0, 0);

  lTarDispo= new QLabel(page,"NoName");
  lTarDispo->setText(i18n("is avaible"));
  gbox->addWidget(lTarDispo, 0, 1);

  QLabel_7= new QLabel(page,"NoName");
  QLabel_7->setText(i18n("The GZIP compressor..."));
  gbox->addWidget(QLabel_7, 1, 0);

  QLabel_8= new QLabel(page,"NoName");
  QLabel_8->setText(i18n("The BZIP2 compressor..."));
  gbox->addWidget(QLabel_8, 2, 0);

  QLabel_9= new QLabel(page,"NoName");
  QLabel_9->setText(i18n("The RAR compressor..."));
  gbox->addWidget(QLabel_9, 3, 0);

  QLabel_10= new QLabel(page,"NoName");
  QLabel_10->setText(i18n("The LHA compressor..."));
  gbox->addWidget(QLabel_10, 4, 0);

  lGzipDispo= new QLabel(page,"NoName");
  lGzipDispo->setText(i18n("is avaible"));
  gbox->addWidget(lGzipDispo, 1, 1);

  lLhaDispo= new QLabel(page,"NoName");
  lLhaDispo->setText(i18n("is avaible"));
  gbox->addWidget(lLhaDispo, 4, 1);

  lRarDispo= new QLabel(page,"NoName");
  lRarDispo->setText(i18n("is avaible"));
  gbox->addWidget(lRarDispo, 3, 1);

  lBzip2Dispo= new QLabel(page,"NoName");
  lBzip2Dispo->setText(i18n("is avaible"));
  gbox->addWidget(lBzip2Dispo, 2, 1);

  QLabel_15= new QLabel(page,"NoName");
  QLabel_15->setText(i18n("The Zip reader..."));
  gbox->addWidget(QLabel_15, 5, 0);

  lUnzipDispo= new QLabel(page,"NoName");
  lUnzipDispo->setText(i18n("is avaible"));
  gbox->addWidget(lUnzipDispo, 5, 1);

  QLbl_16= new QLabel(page,"NoName");
  QLbl_16->setText(i18n("The Arj reader..."));
  gbox->addWidget(QLbl_16, 7, 0);

  lUnarjDispo= new QLabel(page,"NoName");
  lUnarjDispo->setText(i18n("is avaible"));
  gbox->addWidget(lUnarjDispo, 7, 1);
}

/** set all widgets to default settings */
void CPreferences::slotDefault(){
	QFont font;
	
	//Gzip panel
	cmbOGZTauxCompression->setCurrentItem(0);
	cbOGZEcrase->setChecked(true);
	
	//Tar panel
	cbOTCheminAbsolu->setChecked(false);
	cbOTToutChemin->setChecked(false);
	cbOTRecursif->setChecked(true);
//	cbOTEcrase->setChecked(true);
	
	//Directories panel
	rbDDEDMaison->setChecked(true);
	rbDDODMaison->setChecked(true);
	
	//Icon size panel
	rbISFPetit->setChecked(true);
	
	//Reading panel
	rbOLLent->setChecked(true);
	
	//Font panel
	font=QFont::defaultFont();
	font.setFamily("helvetica");
	font.setPointSize(12);
	mFont->setFont(font);
	
	//Date panel
	rbDDLocalisedSmall->setChecked(true);
	
	//Tip of the day panel
	cbOTDDisplayTipofDay->setChecked(true);
	
	//Misc options
	rbMOSMMulti->setChecked(true);
}

/** allow user to select a font for the listview */
void CPreferences::setFontOptions(){
  QVBox *page = addVBoxPage( i18n("Font"), i18n("List font" ),
			     BarIcon("fonts", KIcon::SizeMedium ) );

  QString txt=i18n("font");
  mFont = new KFontChooser( page,txt,false,QStringList(),false,6 );
  mFont->setSampleText( i18n("karchiveur.cpp	80Ko 18:00:12") );
}

/** return the font for the listview */
QFont CPreferences::getFont(){
	return mFont->font();
}

/** set the widgets for the date displayed on the
  * listview */
void CPreferences::setDateOptions(){
  QFrame *page = addPage( i18n("Date"), i18n("Display date"),
			  BarIcon("date", KIcon::SizeMedium ) );

  QGridLayout *gbox = new QGridLayout( page, 6, 4, 5 );

  bgDateDisplay= new QButtonGroup(page,"dateGroupbox");
  bgDateDisplay->setTitle("");
  gbox->addMultiCellWidget(bgDateDisplay, 0, 6, 0, 1);

  QGridLayout *grid1 = new QGridLayout(bgDateDisplay, 3, 0, 15,7 );
  rbDDStandard= new QRadioButton(bgDateDisplay,"NoName");
  rbDDStandard->setText(i18n("Compressors's one"));
  grid1->addWidget(rbDDStandard, 0, 0);

  rbDDLocalisedSmall= new QRadioButton(bgDateDisplay,"NoName");
  rbDDLocalisedSmall->setText(i18n("Localised: 1/1/2000"));
  grid1->addWidget(rbDDLocalisedSmall, 1, 0);

  rbDDLocalisedLarge= new QRadioButton(bgDateDisplay,"NoName");
  rbDDLocalisedLarge->setText(i18n("Localised: Saturday May 2000"));
  grid1->addWidget(rbDDLocalisedLarge, 2, 0);
}

/** return the kind of date that will be displayed in
the listview:
0: standard one (compressor's one)
1: localised 1/1/2000
2: localised: Saturday May 2000 */
int CPreferences::getKindOfDate(){
  if(rbDDStandard->isChecked())
    return 0;
  else if(rbDDLocalisedSmall->isChecked())
    return 1;
  else if(rbDDLocalisedLarge->isChecked())
    return 2;
  return 0;
}

/** set the widgets for the TipofDay tab */
void CPreferences::setTipofDayOptions(){
  QFrame *page = addPage( i18n("Tips"), i18n("Tip of the day"),
			  BarIcon("help", KIcon::SizeMedium ) );
  QVBoxLayout *topLayout = new QVBoxLayout( page, 0, spacingHint() );

  QGridLayout *gbox = new QGridLayout( 3, 0 );
  topLayout->addLayout( gbox );

  cbOTDDisplayTipofDay= new QCheckBox(page,"NoName");
  cbOTDDisplayTipofDay->setText(i18n("Display the tip of the day at startup"));
  gbox->addWidget(cbOTDDisplayTipofDay , 0, 0 );

  topLayout->addStretch(10);
}

/** return true if we'll display the tip of the day
at startup */
bool CPreferences::getDisplayTipofDay(){
	return cbOTDDisplayTipofDay->isChecked();
}

/** set misc options for karchiveur */
void CPreferences::setMiscOptions(){
  QFrame *page = addPage( i18n("Misc"), i18n("Misc options"),
			  BarIcon("configure", KIcon::SizeMedium ) );

  QGridLayout *gbox = new QGridLayout( page, 6, 4, 5 );

  gbMOSelectionMode= new QButtonGroup(page,"NoName");
  gbMOSelectionMode->setTitle(i18n("Selection of list's items"));
  gbox->addMultiCellWidget(gbMOSelectionMode, 0, 6, 0, 1);

  QGridLayout *grid1 = new QGridLayout(gbMOSelectionMode, 2, 0, 15,7 );

  rbMOSMMulti= new QRadioButton(gbMOSelectionMode,"NoName");
  rbMOSMMulti->setText(i18n("Click to add to selection"));
  grid1->addWidget(rbMOSMMulti, 0, 0);

  rbMOSMExtended= new QRadioButton(gbMOSelectionMode,"NoName");
  rbMOSMExtended->setText(i18n("Ctrl+click to add to selection"));
  grid1->addWidget(rbMOSMExtended, 1, 0);
}

/** return 0 if we're in standard Multi selection mode
1 for extended selection mode
(i.e. ctrl+click to select) */
int CPreferences::getSelectionMode(){
	if(rbMOSMMulti->isChecked())
		return 0;
	else
		return 1;
}
#include "cpreferences.moc"
