/***************************************************************************
                          karchiveur.cpp  -  description
                             -------------------
    begin                : mer jun 14 00:26:31 CEST 2000
    copyright            : (C) 2000 by Eric Coquelle
    email                : coquelle@caramail.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

// include files for QT
#include <qdir.h>
#include <qprinter.h>
#include <qpainter.h>

// include files for KDE
#include <kiconloader.h>
#include <kmessagebox.h>
#include <kfiledialog.h>
#include <kmenubar.h>
#include <klocale.h>
#include <kconfig.h>
#include <kfileitem.h>
#include <kopenwith.h>
#include <kstdaction.h>
#include <kstddirs.h>
#include <kurldrag.h>
#include <kedittoolbar.h>
#include <kdebug.h>
#include <math.h>

// application specific includes
#include "karchiveur.h"
#include "karchiveurview.h"
#include "resource.h"


#include <errno.h>
#include <iostream.h>
KarchiveurApp::KarchiveurApp()
{
	archiveobj=new CArchive();

  config=kapp->config();

  ///////////////////////////////////////////////////////////////////
  // call inits to invoke all other construction parts
  initStatusBar();
  initActions();
  initView();
  	
	fichier=new CChoixFichier(this);
	fichier->hide();
	extraction=new CExtraction();
	extraction->hide();

	//Launch the archive browser
	navigateur = new CNavigateur();
	connect( navigateur, SIGNAL(windowwillbeclosed()), this, SLOT(slotViewButineur()));
	//navigateur->exec();
	navigateur->show();
	connect(navigateur->QListBox_1, SIGNAL(selected(const QString&)), this, SLOT(slotOpenFileFromArchiveBrowser(const QString&)));


  readOptions();


  //Additionnal init to the statusbar
	avancement=new KProgress(statusBar(),"barre progression");
	avancement->setGeometry(150,3,200,statusBar()->height()-3);

	led=new CLed(statusBar(),"led");
	led->setLook(KLed::Sunken);
	led->setGeometry(statusBar()->width()-20,9,19,statusBar()->height()-3);

	checkFirstInstall();

	//The tip of the day
	tipofday = new KTipofDay(this);
	if(displaytipofday)
		tipofday->show();

	operation=NOARCHIVE; //No file loaded, nothing to do !
	optionExtraction=1; //Default extraction: in home dir
	optionOuverture=1; //Default opening: in home dir
	recursif=true; //when creating an archive, aslo compress subfolders

	//Initialise for drops
	//The main drop zone is in KarchiveurView (via QListView* liste)
	setAcceptDrops(true);
	connect(view->getWidgetListe(),SIGNAL(faireGlisserDeposer()), this, SLOT(slotDoDragAndDrop()));
	connect(view->getWidgetListe(),SIGNAL(rightButtonPressed ( QListViewItem *, const QPoint &, int )), this, SLOT(slotShowContextualMenu ( QListViewItem *, const QPoint &, int )));
	connect(view->getWidgetListe(),SIGNAL(clicked ( QListViewItem * )), this, SLOT(slotMouseClick ( QListViewItem * )));

	//Initialise the process object:
	process=new CProcessus();
	process->clearArguments();
	connect(process,SIGNAL(receivedStdout(KProcess*, char*, int)),this,SLOT(recueSdtOut(KProcess*,char*,int)));
	connect(process,SIGNAL(processExited (KProcess*)),this,SLOT(finProcessus(KProcess*)));

	//Initialise the find in this archive tool
	find = new CFind();
	connect( find, SIGNAL(findNext(QString)), this, SLOT(slotFindInThisArchive(QString)));
	connect( find, SIGNAL(findClose()), this, SLOT(slotCloseFind()));
	navigateur->exec();
	navigateur->show();

	//Initialise the find archives on disk tool
	searchBox=new CRecherche;
	connect(searchBox->foundarchives, SIGNAL(selected(const QString&)), this, SLOT(slotOpenFileFromArchiveFinder(const QString&)));
	
	 QString directory;
	 directory.sprintf("karchivertmp.%d/", getpid());
	 reptmp = locateLocal( "tmp", directory );
	 
	 if (reptmp.isNull()) 
	   qWarning(i18n("Failed to create temporary "
		    "attachment directory '%2': %1")
		    .arg(strerror(errno))
		    .arg(directory));
	 

	 directory.sprintf("karchivertmp.%d/extract/", getpid());
	 reptmp = locateLocal( "tmp", directory );
	 
	 if (reptmp.isNull()) 
	   qWarning(i18n("Failed to create temporary "
		    "attachment directory '%2': %1")
		    .arg(strerror(errno))
		    .arg(directory));
	 
	 directory.sprintf("karchivertmp.%d/undo/", getpid());
	 reptmpundo = locateLocal( "tmp", directory );
	 
	 if (reptmpundo.isNull())
	   qWarning(i18n("Failed to create temporary "
		    "attachment directory '%2': %1")
		    .arg(strerror(errno))
		    .arg(directory));

	slotViewButineur();
	slotViewButineur();

	connect(archiveobj, SIGNAL(archiveReadEnded()), this, SLOT(slotEndProcess()));
	archiveobj->setWidgetListView(view->getWidgetListe());
	archiveobj->setStatusBar(statusBar());
	archiveobj->setWidgetLed(led);
	archiveobj->setWidgetProgressBar(avancement);
	archiveobj->setArchiveName("");
	archiveobj->setOverwriteFilesWhenExtracting(ecrasergzip);
	archiveobj->setIconSize(16);
	archiveobj->setReadArchiveWithStream(lecturearchiveflot);
	delete archiveobj;
	archiveobj=NULL;

}

KarchiveurApp::~KarchiveurApp()
{

  QString directory;
  directory.sprintf("karchivertmp.%d/", getpid());
  reptmp = locateLocal( "tmp", directory );
  
  process->clearArguments();
  *process << "rm" << "-Rf" <<reptmp;
  process->start(KProcess::Block);
  if(archiveobj!=NULL)
    delete archiveobj;
  kapp->clipboard()->clear();
}

void KarchiveurApp::resizeEvent( QResizeEvent * )
{
	int largeur;
	int hauteur;

	largeur=width()-1;
	hauteur=height();

	if(viewStatusbar->isChecked())
		hauteur-=75;
	else
		hauteur-=55;
	view->getWidgetListe()->setGeometry(0,0,largeur,hauteur);
	led->setGeometry(statusBar()->width()-20,4,statusBar()->height()-5,statusBar()->height()-5);
	avancement->setGeometry(width()/3,3,width()/3,statusBar()->height()-3);
}

void KarchiveurApp::initStatusBar()
{
  ///////////////////////////////////////////////////////////////////
  // STATUSBAR
  // TODO: add your own items you need for displaying current application status.
  statusBar()->insertItem(i18n(IDS_STATUS_DEFAULT), ID_STATUS_MSG);
}

void KarchiveurApp::initView()
{
  ////////////////////////////////////////////////////////////////////
  // create the main widget here that is managed by KTMainWindow's view-region and
  // connect the widget to your document to display document contents.

  view = new KarchiveurView(this);
	setCentralWidget(view);
}

void KarchiveurApp::saveOptions()
{
  config->setGroup("General Options");
  config->writeEntry("Geometry", size());
  config->writeEntry("Show Toolbar", viewToolbar->isChecked());
  config->writeEntry("Show Statusbar",viewStatusbar->isChecked());
  config->writeEntry("Show Navigateur",viewArchiveBrowser->isChecked() );
  config->writeEntry("ToolBarPos", (int) toolBar("mainToolBar")->barPos());
  fileOpenRecent->saveEntries(config,"Recent Files");

	config->setGroup("LASTDIRS");
	config->writeEntry("Extraction",extraction->getCheminAcces());
	config->writeEntry("Ouverture",fichier->getCheminAcces());
}


void KarchiveurApp::readOptions()
{

  config->setGroup("General Options");

  // bar status settings
  bool bViewToolbar = config->readBoolEntry("Show Toolbar", true);
  viewToolbar->setChecked(bViewToolbar);
  slotViewToolBar();

  bool bViewStatusbar = config->readBoolEntry("Show Statusbar", true);
  viewStatusbar->setChecked(bViewStatusbar);
  slotViewStatusBar();

  bool bViewNavigateur = config->readBoolEntry("Show Navigateur", true);
	viewArchiveBrowser->setChecked(bViewNavigateur);
	slotViewButineur();

  // bar position settings
  KToolBar::BarPosition toolBarPos;
  toolBarPos=(KToolBar::BarPosition) config->readNumEntry("ToolBarPos", KToolBar::Top);
  toolBar("mainToolBar")->setBarPos(toolBarPos);

  // initialize the recent file list
  fileOpenRecent->loadEntries(config,"Recent Files");

  QSize size=config->readSizeEntry("Geometry", new QSize(640,480));
  if(!size.isEmpty())
  {
    resize(size);
  }

  config->setGroup("OPTIONS");
  incluretoutchemin=config->readBoolEntry("ToutChemin",false);
  recursif=config->readBoolEntry("Recursif",true);
	ecrasergzip=config->readBoolEntry("EcraserGzip",true);
	cheminabsolu=config->readBoolEntry("CheminAbsolu",false);
	switch(config->readNumEntry("LectureParFlot",0))
	{
		case 0:
		  lecturearchiveflot=true;
			archiveobj->setDisplayIcons(true);
		  break;
	 	case 1:
		 	lecturearchiveflot=false;
			archiveobj->setDisplayIcons(true);
		 	break;
		case 2:
			//Don't display icons for archive dontent display.
			lecturearchiveflot=false;
			archiveobj->setDisplayIcons(false);
			break;
	}

	if(config->readNumEntry("SelectionMode",0)==0)
	  view->getWidgetListe()->setSelectionMode(QListView::Multi);
	 else
	 	view->getWidgetListe()->setSelectionMode(QListView::Extended);

	// For the compress rate, in the config file 1 = min compress
	tauxCompression=config->readNumEntry("TauxCompression",9);
	archiveobj->setCompressRate(tauxCompression);

	config->setGroup("LASTDIRS");
	QString cheminExtraction=config->readEntry("Extraction",QDir::homeDirPath());
	QString cheminOuverture=config->readEntry("Ouverture",QDir::homeDirPath());

	optionExtraction=config->readNumEntry("OptionExtraction",1);
	optionOuverture=config->readNumEntry("OptionOuverture",1);
	fichier->setCheminAcces(cheminOuverture,optionOuverture);
	fichier->setForInitPreviousPath(cheminOuverture);
	navigateur->setChemin(fichier->getCheminAcces());
	extraction->setCheminAcces(cheminExtraction, optionExtraction);

	config->setGroup("DATE");
	archiveobj->setKindOfDate(config->readNumEntry("KindOfDate",0));

	config->setGroup("TipOfTheDay");
	displaytipofday=config->readBoolEntry("show_tod", true);

	archiveobj->setRecursiveMode(recursif);
}

void KarchiveurApp::saveProperties(KConfig *)
{
}


void KarchiveurApp::readProperties(KConfig* _cfg)
{
  QString filename = _cfg->readEntry("filename", "");
  bool modified = _cfg->readBoolEntry("modified", false);
  if(modified)
  {
    bool canRecover;
    QString tempname = kapp->checkRecoverFile(filename, canRecover);

    if(canRecover)
    {
      QFileInfo info(filename);
      QFile::remove(tempname);
    }
  }
  else
  {
    if(!filename.isEmpty())
    {
    }
  }
}

bool KarchiveurApp::queryClose()
{
  return true;
}

bool KarchiveurApp::queryExit()
{
  saveOptions();
  return true;
}

/////////////////////////////////////////////////////////////////////
// SLOT IMPLEMENTATION
/////////////////////////////////////////////////////////////////////

void KarchiveurApp::slotFileNewWindow()
{
  slotStatusMsg(i18n("Opening a new application window..."));

  KarchiveurApp *new_window= new KarchiveurApp();
  new_window->show();

  slotStatusMsg(i18n(IDS_STATUS_DEFAULT));
}

void KarchiveurApp::slotFileNew()
{
  slotStatusMsg(i18n("Creating new document..."));
	setCaption("");

  view->getWidgetListe()->clear();
  operation=NOARCHIVE;

	if(archiveobj!=NULL)
		delete archiveobj;
	archiveobj=NULL;

  slotStatusMsg(i18n(IDS_STATUS_DEFAULT));
}

void KarchiveurApp::slotFileOpen()
{
  slotStatusMsg(i18n("Opening archive..."));
	led->setColor(QColor("red"));

	if(fichier->openArchive()==NULL)
	{
			slotStatusMsg(IDS_STATUS_DEFAULT);
			return;
	}

	displayArchiveContent();

  displayMessageArchiveSize();
}

void KarchiveurApp::slotFileOpenRecent(const KURL& url)
{
  QString archiveOpenName=openRemoteFile(url);
  if(!archiveOpenName.isEmpty())
    {
      fichier->setNomArchive(archiveOpenName);
      navigateur->setChemin(fichier->getCheminAcces());
      
      displayArchiveContent();
    }
}

void KarchiveurApp::slotFileOpenRecent(int id_)
{
  fichier->setNomArchive(recentFiles.at(recentFilesMenu->indexOf(id_)));
	navigateur->setChemin(fichier->getCheminAcces());
	displayArchiveContent();
}

/* This method will convert the archive format*/
void KarchiveurApp::slotFileSaveAs()
{
  led->setColor(QColor("red"));
  slotStatusMsg(i18n("Convert archive format..."));
  if(operation!=NOARCHIVE)
    {
      QString repertoire;
      QString tmp=NULL;
      repertoire=reptmp;
			
      //We will extract the archive to a temp folder, so let's clean it
      process->clearArguments();
      *process << "rm" << "-fR" <<repertoire;
      process->start(KProcess::Block);
      process->clearArguments();
      *process << "mkdir" << reptmp;
      process->start(KProcess::Block);
      operation=CONVERSION;
      avancement->setValue(0);
      //avancement.setTexte(i18n("Extracting files"));
      avancement->setRange(0,view->getWidgetListe()->childCount());
      archiveobj->extractArchive(reptmp,0,tmp);
      led->setColor(QColor("green"));
    }
  else
    {
      slotStatusMsg(i18n("First open an archive !"));
      led->setColor(QColor("orange"));
      return;
    }
  slotStatusMsg(IDS_STATUS_DEFAULT);
}

void KarchiveurApp::slotFileClose()
{
  slotStatusMsg(i18n("Closing file..."));

  close();
  //slotStatusMsg(i18n(IDS_STATUS_DEFAULT));
}

void KarchiveurApp::slotFilePrint()
{
  slotStatusMsg(i18n("Printing..."));

  QPrinter printer;

  if (printer.setup(this))
  {
    view->print(&printer);
  }

  slotStatusMsg(i18n(IDS_STATUS_DEFAULT));
}

void KarchiveurApp::slotFileQuit()
{
  slotStatusMsg(i18n("Exiting..."));
  saveOptions();
  close();
}

void KarchiveurApp::slotViewToolBar()
{
  slotStatusMsg(i18n("Toggle the toolbar..."));
  ///////////////////////////////////////////////////////////////////
  // turn Toolbar on or off
  if(!viewToolbar->isChecked())
  {
    toolBar("mainToolBar")->hide();
  }
  else
  {
    toolBar("mainToolBar")->show();
  }

  slotStatusMsg(i18n(IDS_STATUS_DEFAULT));
}

void KarchiveurApp::slotViewStatusBar()
{
  slotStatusMsg(i18n("Toggle the statusbar..."));
  ///////////////////////////////////////////////////////////////////
  //turn Statusbar on or off
  if(!viewStatusbar->isChecked())
  {
    statusBar()->hide();
  }
  else
  {
    statusBar()->show();
  }
	repaint();
  slotStatusMsg(i18n(IDS_STATUS_DEFAULT));
}

void KarchiveurApp::slotStatusMsg(const QString &text)
{
  ///////////////////////////////////////////////////////////////////
  // change status message permanently
  statusBar()->clear();
  statusBar()->changeItem(text, ID_STATUS_MSG);
}


void KarchiveurApp::slotStatusHelpMsg(const QString &text)
{
  ///////////////////////////////////////////////////////////////////
  // change status message of whole statusbar temporary (text, msec)
  statusBar()->message(text, 2000);
}

/** displays a window and search recursively
	* for archives in a directory */
void KarchiveurApp::slotFileSearch(){
	searchBox->setPath(fichier->getCheminAcces(), optionOuverture);
	searchBox->exec();
}

/** Make all or some files to be extracted */
void KarchiveurApp::slotExtractArchive(){
  QString tmp=NULL;
  if(operation==NOARCHIVE)
    {
      slotStatusMsg(i18n("First open an archive !"));
      led->setColor(QColor("orange"));
      return;
    }
  
  slotStatusMsg(i18n("Extract files..."));
  led->setColor(KLed::red);
  
  extraction->afficheChemin();
  if(extraction->exec())
  {
      operation=EXTRACTING;
      archiveobj->extractArchive(extraction->getCheminAcces(),extraction->toutExtraire(),tmp);
  }
  else
  {
		led->setColor(KLed::green);
		displayMessageArchiveSize();
  }
}

/** add some files to the archive */
void KarchiveurApp::slotAddFiles(){
 	CAjoutFichiers ajoutfichiers;

  slotStatusMsg(i18n("Add files..."));
  led->setColor(QColor("red"));

	if(ajoutfichiers.exec()==0)
	{
		if(operation!=-1)
		{
			//An archive is already loaded
			operation=ADDING;
			QString archive;
			int actionfichiers;
			bool enleverfichiers;
			QStrList fichiersAjout;

			archive=fichier->getNomFichier();
			fichiersAjout=ajoutfichiers.getFichiers();
			actionfichiers=ajoutfichiers.getAction();
			enleverfichiers=ajoutfichiers.getEnleveFichiers();

			addToArchive(archive,fichiersAjout,enleverfichiers,actionfichiers);
		}
		else
		{
			//No, so create an archive
			int kindofCompressor;
			KFileDialog boiteSauver(i18n("Create archive"),"*",this,"boite creer archive",false);

			//boiteSauver.setName(i18n("Create archive"));
			QString nomarchive=boiteSauver.getSaveFileName();
			fichier->setNomArchive(nomarchive);

			if(fichier->findKindOfCompressor()==1)
			{
				slotStatusMsg(i18n("Unknown archive type"));
				return;
			}
			operation=-2;
			kindofCompressor=fichier->getKindOfCompressor();
			if(strcmp(fichier->getProgrCompresseur(),"tar")==0)
			{
				avancement->setValue(0);
				avancement->setRange(0,5);
				if(kindofCompressor==TARGZ)
					archiveobj=new CTarGz();
				else if(kindofCompressor==TARBZ2)
					archiveobj=new CTarBz2();
				else
					archiveobj=new CTar();
				//avancement->setTexte(i18n("Creating archive"));
				avancement->advance(1);
				archiveobj->createArchive(nomarchive,ajoutfichiers.getFichiers());
				avancement->advance(1);
				displayArchiveContent();
			}
			else
			{
			  KMessageBox::sorry( this,
					      i18n("Files can be added only in\n"
						   ".tar, .tar.gz, .tar.bz2"),
					      "karchiver" );
			  led->setColor(QColor("orange"));
			  return;
			}
		}
	}

	led->setColor(KLed::green);
	displayMessageArchiveSize();
}

/** display a file using its associated program */
void KarchiveurApp::slotDisplayFile(){
  slotStatusMsg(i18n("View this file..."));
	led->setColor(QColor("red"));

	if(operation!=NOARCHIVE)
	{
		QString txt;
		KFileOpenWithHandler* owh;
		operation=VISUALISATION;
		QListViewItem *li=view->getWidgetListe()->currentItem();

		if(li==0)
		{
		  slotStatusMsg(i18n("First select a file"));
		  led->setColor(QColor("orange"));
		  return;
		}
		//Get the path of selected element
		txt=li->text(6);
		if(!txt.isEmpty())
		{
			txt=txt.remove(0,1);
			txt+=li->text(0);
		}
		else
		{
			txt=li->text(0);
			txt=txt.remove(0,1);
		}
		archiveobj->extractArchive(reptmp, 9,txt);
		waitpid(process->getPid(),NULL,WNOHANG);
		if((fichier->getKindOfCompressor()==GZIP)||(fichier->getKindOfCompressor()==BZIP2))
		{
			txt=txt.right(txt.length()-txt.findRev('/')-1);
		}
		txt=reptmp+txt;

		KFileItem fi(0,0,KURL(txt));
		owh=new KFileOpenWithHandler();
		fi.run();
	}

  slotStatusMsg(IDS_STATUS_DEFAULT);
  led->setColor(QColor("green"));
}

/** display all selected files */
void KarchiveurApp::slotDisplaySelectedFiles(){
  slotStatusMsg(i18n("View all selected files..."));

	if(operation!=NOARCHIVE)
	{
		int i;
		int k;
		QListViewItem *li=view->getWidgetListe()->firstChild();
		QString txt;
		KFileOpenWithHandler* owh;
		KFileItem *fi;

		if(li==0)
		{
		  slotStatusMsg(i18n("First select a file"));
		  led->setColor(QColor("orange"));
			return;
		}

		led->setColor(QColor("red"));

    owh=new KFileOpenWithHandler();
		k=view->getWidgetListe()->childCount();

		for(i=0;i<k;i++)
		{
			if(view->getWidgetListe()->isSelected(li))
			{
				txt=li->text(6);
				txt=txt.remove(0,1);
				txt+=li->text(0);
				archiveobj->extractArchive(reptmp, 9, txt);
				txt=reptmp+txt;
				fi=new KFileItem(0,0,KURL(txt));
				fi->run();
			}
			if(i!=k)
				li=li->nextSibling();
		}
	}

	led->setColor(QColor("green"));
  slotStatusMsg(IDS_STATUS_DEFAULT);
}

/** remove from archive all selected files */
void KarchiveurApp::slotRemoveSelectedFiles(){
  led->setColor(QColor("red"));
  slotStatusMsg(i18n("Deleting selected files..."));
  kdDebug()<<("IN:slotRemoveSelectedFiles\n");
	if(operation!=NOARCHIVE)
	{
		int i;
		int k;
		QListViewItem *li=view->getWidgetListe()->firstChild();
		QString txt;
		QStrList strlist;

		k=view->getWidgetListe()->childCount();
		for(i=0;i<k;i++)
		{
			if(view->getWidgetListe()->isSelected(li))
			{
				txt=li->text(6);
				if(txt=="")
				{
					txt=li->text(0);
					txt=txt.remove(0,1);
				}
				else
				{
					txt=txt.remove(0,1);
					txt+=li->text(0);
				}
				strlist.append(txt);
				kdDebug()<<QString("slotRemove:%1*").arg(txt)<<endl;
			}
			if(i!=k)
				li=li->nextSibling();
		}
		if(KMessageBox::questionYesNoList(this, i18n("Do you really want to delete these files?"),
																						QStringList::fromStrList(strlist),
																						i18n("KArchiver"))
			==KMessageBox::No)
		{
			led->setColor(QColor("green"));
		  slotStatusMsg(IDS_STATUS_DEFAULT);
			return;
		}

		archiveobj->removeFilesFromArchive(strlist);
		operation=STANDBY;
	}

	displayArchiveContent();
	led->setColor(QColor("green"));
	displayMessageArchiveSize();
}

/** display some infos about the current archive */
void KarchiveurApp::slotDisplayArchiveInfos(){
	QListViewItem *item;
	QString txt;
	int total;
	int moyenne;
	int ecarttype;
	int taux;
	int taillearchive;
	int nbrefichiers;
	int i;

	if((fichier->getKindOfCompressor()==GZIP)||(fichier->getKindOfCompressor()==BZIP2)||(operation==NOARCHIVE))
		return;

	total=0;
	ecarttype=0;

	item=NULL;
	item=view->getWidgetListe()->firstChild();
	nbrefichiers= view->getWidgetListe()->childCount();
	while(item)
	{
		txt=item->text(1);
		i=txt.toInt();
		total+=i;
		ecarttype+=i*i/nbrefichiers;
		item=item->nextSibling();
	}

	taillearchive=tailleArchive(fichier->getNomFichier());
	moyenne=total/nbrefichiers;
	taux=100-100*taillearchive/total;

	ecarttype-=moyenne;
	ecarttype=(int)sqrt((float)ecarttype);
	CInfos boiteinfos;
	boiteinfos.initialise(nbrefichiers, taillearchive, total, moyenne, taux, ecarttype);
	boiteinfos.exec();
}

/** allow user to set the options */
void KarchiveurApp::slotSetOptions(){
	CPreferences prefs;

	if(prefs.exec())
	{
		recursif=prefs.getRecursif();
		archiveobj->setRecursiveMode(recursif);
		cheminabsolu=prefs.getCheminAbsolu();
		ecrasergzip=prefs.getEcraserGzip();
		archiveobj->setOverwriteFilesWhenExtracting(ecrasergzip);
		incluretoutchemin=prefs.getToutChemin();
		tauxCompression=prefs.getTauxCompression();
		optionExtraction=prefs.getTypeRepertoireExtraction();
		optionOuverture=prefs.getTypeRepertoireOuverture();
		archiveobj->setIconSize(prefs.getTailleIconeListe());
		fichier->setOptionOuverture(optionOuverture);
		extraction->setOptionExtraction(optionExtraction);
		view->getWidgetListe()->setFont(prefs.getFont());
		archiveobj->setKindOfDate(prefs.getKindOfDate());
		switch(prefs.getLitParFlot())
		{
			case 0:
				lecturearchiveflot=true;
				archiveobj->setDisplayIcons(true);
				break;
			case 1:
				lecturearchiveflot=false;
				archiveobj->setDisplayIcons(true);
				break;
			case 2:
				lecturearchiveflot=false;
				archiveobj->setDisplayIcons(false);
				break;
		}
		archiveobj->setCompressRate(prefs.getTauxCompression());
		if(prefs.getSelectionMode()==0)
			view->getWidgetListe()->setSelectionMode(QListView::Multi);
		else
			view->getWidgetListe()->setSelectionMode(QListView::Extended);
	}
}

/** Split/Unsplit the archive in/from small parts */
void KarchiveurApp::slotDiskCutUncut(){
	CWizardStep2Cutt wiz_cutt;

	if(wiz_cutt.exec())
	{
		if(wiz_cutt.rep!=NULL)
			doCuttUnCutt(wiz_cutt.rep);
	}
	return;
}

/** display or not the archive browser */
void KarchiveurApp::slotViewButineur(){
	if(viewArchiveBrowser->isChecked())
	{
		navigateur->show();
	}
	else
		navigateur->hide();
}

/** this method will launch the right compressor
to list the file contents
It will then launch the right method to display
the content in the listview */
void KarchiveurApp::displayArchiveContent()
{
	QString message;
	QString nom;
	int kindofCompressor;

	led->setColor(QColor("red"));

	if(fichier->findKindOfCompressor()==1)
	{
		slotStatusMsg(i18n("Unknown archive type"));
		operation=NOARCHIVE;
		led->setColor(QColor("orange"));
		return;
	}
	
/*	if(fichier->isCurrentCompressorAvaible())
	{
		KMessageBox::sorry( this,
											i18n("The required compressor is not installed\non your system. See options->compressors to\ncheck which one you have !"),
											i18n("karchiver: compressor not found !")
											);
	}
*/
	nom=fichier->getNomFichier();
	setCaption(nom);

  fileOpenRecent->addURL(KURL(fichier->getNomFichier()));
	navigateur->setChemin(fichier->getCurrentArchiveDirectory());

	view->getWidgetListe()->clear();
	for(int i=0;i<7;i++)
		view->getWidgetListe()->setColumnWidth(i,50);
	operation=READING;

	kindofCompressor=fichier->getKindOfCompressor();

	if(archiveobj!=NULL)
		delete archiveobj;

	switch(kindofCompressor)
	{
		case TARGZ:
			archiveobj=new CTarGz();
			break;
		case TARBZ2:
			archiveobj=new CTarBz2();
			break;
		case TAR:
			archiveobj=new CTar();
			break;
		case ZIP:
			archiveobj=new CZip();
			break;
		case ARJ:
			archiveobj=new CArj();
			break;
		case LHA:
			archiveobj=new CLha();
			break;
		case RAR:
			archiveobj=new CRar();
			break;
		case GZIP:
			archiveobj=new CGz();
			break;
		case BZIP2:
			archiveobj=new CBz2();
			break;
	}
	connect(archiveobj, SIGNAL(archiveReadEnded()), this, SLOT(slotEndProcess()));
	archiveobj->setReadArchiveWithStream(lecturearchiveflot);
	archiveobj->setArchiveName(fichier->getNomFichier());
	archiveobj->displayArchiveContent();
}

/** return the size in bytes of the current archive */
int KarchiveurApp::tailleArchive(const QString & nomArchive){
	FILE *F1;
	long tailleArchive;

	tailleArchive=0;
	if ((F1 = fopen(nomArchive, "rb")) == NULL)
		return 0;
	fseek(F1, 0, SEEK_END);
	tailleArchive=ftell(F1);
	fclose(F1);
	return tailleArchive;
}

/** Launch the wizard */
void KarchiveurApp::slotWizard(){
	//First, we extract all files to reptmp
	//Once process exits to slotEndProcess, we launch the wizard
	if(operation!=-1)
	{
		QString repertoire;
		QString tmp=NULL;

		led->setColor(KLed::red);
		slotStatusMsg(i18n("Running wizard"));

		repertoire=reptmp;
		process->clearArguments();
		*process << "rm" << "-fR" <<repertoire;
		process->start(KProcess::Block);
		process->clearArguments();
		*process << "mkdir" << reptmp;
		process->start(KProcess::Block);

		operation=SCRIPT;
		archiveobj->extractArchive(reptmp,0,tmp);
	}
}

/** The current process ended */
void KarchiveurApp::finProcessus(KProcess*){
	int kindofCompressor=fichier->getKindOfCompressor();

	if(operation==READING)
	{
		led->setColor(QColor("green"));
		displayMessageArchiveSize();
		operation=STANDBY;
		return;
	}
	else if(operation==EXTRACTING)
	{
		if((kindofCompressor==GZIP)||(kindofCompressor==BZIP2))
		{
			fclose(fichierIO);
			led->setColor(KLed::green);
		}
		avancement->setValue(0);
		operation=STANDBY;
	}
	else if(operation==ADDCREATE)
		{
			displayArchiveContent();
			avancement->setValue(0);
			fclose(fichierIO);
		}
	else if(operation==VISUALISATION)
	{
		if((kindofCompressor==GZIP)||(kindofCompressor==BZIP2))
			fclose(fichierIO);
	}
	displayMessageArchiveSize();
}


/** we recive some informations through the standard
output of the process */
void KarchiveurApp::recueSdtOut(KProcess *, char *buffer, int length){

	if((operation==CONVERSION)||(operation==SCRIPT))
	{
		int i=0;
		if(fichier->getKindOfCompressor()==ZIP)
		{
			compteur++;
			if(compteur%2==0)
				avancement->advance( 1 );
		}
		//A little animation for waiting gzip and bzip2 files to uncompress
		else if((fichier->getKindOfCompressor()==GZIP)||(fichier->getKindOfCompressor()==BZIP2))
		{
			compteur++;
			fwrite(buffer,sizeof(char),length,fichierIO);
			QString s=i18n("Running compressor...");
  		switch(compteur/4%8)
  		{
  			case 0:
  				s+="-";
  				break;
  			case 1:
  				s+="/";
  				break;
  			case 2:
  				s+="|";
  				break;
  			case 3:
  				s+="\\";
  				break;
  			case 4:
  				s+="-";
  				break;
  			case 5:
  				s+="/";
  				break;
  			case 6:
  				s+="|";
  				break;
  			case 7:
  				s+="\\";
  				break;
   		}
  		slotStatusMsg(s);
 		}
 		//For all other formats, use the progress bar
 		else
 		{
			QString inter;
			inter=buffer;
			inter=inter.left(length);

 			while((i=inter.find('\n',i+1))!=-1){
 				avancement->advance(1);
 			}
 		}
	}
	//We add files to an archive or we create one
	else if(operation==ADDCREATE)
	{
		if((fichier->getKindOfCompressor()==GZIP)||(fichier->getKindOfCompressor()==BZIP2))
		{
			fwrite(buffer,sizeof(char),length,fichierIO);
		}
		else
		{
			avancement->advance(1);
			//if(avancement.estAnnule())
			//	process->kill();
		}
	}
	//We see a gzip or bzip2 archive
	else if(operation==VISUALISATION)
	{
		if((fichier->getKindOfCompressor()==GZIP)||(fichier->getKindOfCompressor()==BZIP2))
		{
			fwrite(buffer,sizeof(char),length,fichierIO);
		}
	}
}

/** Add some files to the archive
@fichiersAjout : list of files to add
@enlevefichiers : remove or not those files from disk
@actionfichiers : 0=mode append and replace files, 1=mode update files
@cheminRelatif : include only filenames, without their path */
void KarchiveurApp::addToArchive(const QString &, QStrList fichiersAjout, bool enlevefichiers, int actionfichiers,const char* cheminRelatif){
	archiveobj->addFilesToArchive( fichiersAjout,enlevefichiers,actionfichiers,cheminRelatif);
	displayArchiveContent();
}

/** Compress with the compressor compresseur the archive nomArchive*/
void KarchiveurApp::compressCreatedArchive(const QString & compresseur, const QString & nomArchive){
//printf("ComprArchCree:%s",nomArchive);
	process->clearArguments();
	//gzip et bzip2 acceptent -f
	*process << compresseur;
	if(((strcmp(compresseur,"gzip")==0)&&ecrasergzip)||((strcmp(compresseur,"bzip2")==0)&&ecrasergzip))
		*process << "-f";
	*process << nomArchive;
	process->start(KProcess::Block);
}

/** Remove the files fichiersAEffacer from archive archive */
void KarchiveurApp::removeFilesFromArchive(const QString &, QStrList fichiersAEffacer){
	if(operation==NOARCHIVE)
	{
		led->setColor(QColor("orange"));
		return;
	}

	led->setColor(QColor("red"));

	archiveobj->removeFilesFromArchive(fichiersAEffacer);

	kdDebug()<<("Will display archiveliste\n");
	displayArchiveContent();
	kdDebug()<<("removeFilesFromArchive:OK\n");
	led->setColor(QColor("green"));
}

/** Open the archive nomArchive, which was selected
from the archive browser */
void KarchiveurApp::slotOpenFileFromArchiveBrowser(const QString& nomArchive){
	QString str;

	str=fichier->getCurrentArchiveDirectory();
	str+="/";
	str+=nomArchive;
	fichier->setNomArchive(str);
	displayArchiveContent();
}

/** Open the archive nomArchive, which was found
using the search tool */
void KarchiveurApp::slotOpenFileFromArchiveFinder(const QString& nomArchive){
	QString nom;
	led->setColor(KLed::red);

	nom=nomArchive;
	fichier->setCheminAcces(nom.left(nom.findRev('/')));
	navigateur->setChemin(fichier->getCheminAcces());
	fichier->setNomArchive(nomArchive);
	displayArchiveContent();
}

/** Open the archive nomArchive, which was sent
by the command line */
void KarchiveurApp::openFileFromCommandLine(QStrList names){
	openFilesDropPaste(QStringList::fromStrList(names));
}

/** Convert an archive to type, in the directory repconv */
void KarchiveurApp::doArchiveConversion(int type, const QString & repconv){
	int kindofCompressor;
	QStrList listeFichiers;
	QStrList listeKDE;
	QString s;
	QString archiveOriginale;
	QString nomarchive;
	QString namewithoutextension;
	QDir repertoire=QDir::root();

	repertoire.cd(reptmp);
	archiveOriginale=fichier->getNomFichier();
	kindofCompressor=fichier->getKindOfCompressor();
	listeFichiers=repertoire.encodedEntryList();
	for (QString f = listeFichiers.first(); f; f = listeFichiers.next())
	  {
	    if((strcmp(f,".")==0)||(strcmp(f,"..")==0))
	      listeFichiers.remove();
	    else
	      listeKDE.append(f);
	  }

	namewithoutextension=fichier->getNomFichier();
	kdDebug()<<QString("Analyse of:%1, type:%2,reptmp:%3*").arg(namewithoutextension).arg(type).arg(repconv)<<endl;
	//We remove the extension of the archive name
	if(strcmp(fichier->getProgrCompresseur(),"tar")==0)
	{
		if(namewithoutextension.right(4).lower()==".tgz")
			namewithoutextension.truncate(namewithoutextension.length()-4);
		else if(namewithoutextension.right(6).lower()==".tar.z")
			namewithoutextension.truncate(namewithoutextension.length()-6);
		else if(kindofCompressor==TARGZ)
			namewithoutextension.truncate(namewithoutextension.length()-7);
		else if(kindofCompressor==TARBZ2)
			namewithoutextension.truncate(namewithoutextension.length()-8);
		else if(kindofCompressor==TAR)
			namewithoutextension.truncate(namewithoutextension.length()-4);
	}
	else if(kindofCompressor==GZIP)
		namewithoutextension.truncate(namewithoutextension.length()-3);
	else if(kindofCompressor==BZIP2)
		namewithoutextension.truncate(namewithoutextension.length()-4);
	else if((kindofCompressor==ZIP)||(kindofCompressor==RAR)||(kindofCompressor==LHA)||(kindofCompressor==ARJ))
		namewithoutextension.truncate(namewithoutextension.length()-4);

	switch(type)
	{
		case 0:
			namewithoutextension+=".tar";
			break;
		case 1:
			namewithoutextension+=".tar.gz";
			break;
		case 2:
			namewithoutextension+=".tar.bz2";
			break;
		case 3:
			namewithoutextension+=".zip";
			break;
		case 4:
			namewithoutextension+=".gz";
			break;
		case 5:
			namewithoutextension+=".bz2";
			break;
		case 6:
			namewithoutextension+=".rar";
			break;
		case 7:
			namewithoutextension+=".lha";
			break;
	}

	nomarchive=namewithoutextension;
	//nomarchive will contain the archive's name without path
	nomarchive=nomarchive.right(nomarchive.length()-nomarchive.findRev('/'));

	nomarchive=repconv+nomarchive;

	operation=-2;
	fichier->setNomArchive(nomarchive);
  fichier->findKindOfCompressor();
  kindofCompressor=fichier->getKindOfCompressor();

	if(strcmp(fichier->getProgrCompresseur(),"tar")==0)
	{
		if(kindofCompressor==TARGZ)
		{
			archiveobj=new CTarGz();
		}
		else if(kindofCompressor==TARBZ2)
		{
			archiveobj=new CTarBz2();
		}
		else if(kindofCompressor==TAR)
		{
			archiveobj=new CTar();
		}
		archiveobj->createArchive(nomarchive,listeKDE,reptmp);
		displayArchiveContent();
	}
	else if(kindofCompressor==ZIP)
	{
		archiveobj=new CZip();
		archiveobj->createArchive(nomarchive,listeKDE,reptmp);
	}
	else if(kindofCompressor==RAR)
	{
		archiveobj=new CRar();
		archiveobj->createArchive(nomarchive,listeKDE,reptmp);
	}
	else if(kindofCompressor==LHA)
	{
		archiveobj=new CLha();
		archiveobj->createArchive(nomarchive,listeKDE,reptmp);
	}
	else
	{
		// We convert a .gz or .bz2 archive
  	namewithoutextension=namewithoutextension.left(namewithoutextension.findRev('.'));
		namewithoutextension=namewithoutextension.right(namewithoutextension.length()-namewithoutextension.findRev('/')-1);
		namewithoutextension=reptmp+namewithoutextension;

		process->clearArguments();
		QString pr;
		QString ext;
		if(nomarchive.right(3).lower()==".gz")
		{
			kindofCompressor=GZIP;
			pr="gzip";
			ext=".gz";
		}
		else if(nomarchive.right(4).lower()==".bz2")
		{
			kindofCompressor=BZIP2;
			pr="bzip2";
			ext=".bz2";
		}
		else
		{
	  	KMessageBox::sorry( this,
			i18n("Archives can only be converted in\n"
			".tar, .tar.gz, .tar.bz2, .zip, .rar, .lha"), "karchiver" );
			led->setColor(QColor("orange"));
			return;
		}
		*process << pr;
		*process << "-f";
		*process << namewithoutextension;
		process->start(KProcess::Block);
		process->clearArguments();
		pr="mv";
		*process << pr;
		namewithoutextension+=ext;
		*process << namewithoutextension;
		*process << nomarchive;
		process->start(KProcess::Block);
	}
	s=i18n("Delete : %1 ?").arg(archiveOriginale);
	if(KMessageBox::questionYesNo(this, s, "karchiver")==1)
	{
		process->clearArguments();
		*process << "rm" << "-f" <<archiveOriginale;
		process->start(KProcess::Block);
	}
	led->setColor(QColor("green"));
}

/** split or unsplit the archive nom */
void KarchiveurApp::doCuttUnCutt(QString nom){
	led->setColor(QColor("red"));

	cuttFichier objCoupe;
	QString str;

	if(nom.right(3)==".01")
	{
		nom=nom.left(nom.length()-3);
		if(objCoupe.Debut(nom,1)==0)
		{
			operation=-1;
			fichier->setNomArchive(nom);
			displayArchiveContent();
		}
	}
	else
	{
		str=fichier->getNomFichier();
  		str=str.right(str.length()-str.findRev('/')-1);
		if(nom.right(1)!="/")
			nom+="/";
		str=nom+str;

		objCoupe.Debut(str,0,fichier->getNomFichier());
		
		nom=i18n("archive splitted in %1.01...").arg(fichier->getNomFichier());
		KMessageBox::information(this,nom,i18n("KArchiver"));
	}
	led->setColor(QColor("green"));
}

/** Download a net file and returns its local filename*/
QString KarchiveurApp::openRemoteFile( const KURL& url){
	QString fichierlocal;

  fichierlocal=QDir::homeDirPath();
  fichierlocal+="/";
  fichierlocal+=url.fileName(true);
  kdDebug()<<QString("local file:%1*%2*").arg(fichierlocal).arg(url.prettyURL())<<endl;
  if ( url.isMalformed() )
  {
      KMessageBox::error(this, i18n("Bad url !"));
      led->setColor(QColor("orange"));
      return NULL;
  }

  if (KIO::NetAccess::download(url, fichierlocal) == false)
  {
    // this needs to be handled within KIO sooooon
    KMessageBox::error(this, i18n("Cannot download file!"));
    led->setColor(QColor("orange"));
    return NULL;
  }

  return fichierlocal;
}

/** Displays in the status bar the size of the archive */
void KarchiveurApp::displayMessageArchiveSize(){
	QString message;

 	message=IDS_STATUS_DEFAULT;
 	if(fichier->getNomFichier()!=NULL)
	  message=QString("%1  -  %2 %3").arg(IDS_STATUS_DEFAULT).arg(tailleArchive(fichier->getNomFichier())).arg(i18n("bytes"));
   slotStatusMsg(message);
}

/** A drop will occur, so we have to allow it */
void KarchiveurApp::dragEnterEvent(QDragEnterEvent* e){
	e->accept(QUriDrag::canDecode(e));
}

/** someone has dropped something ! */
void KarchiveurApp::dropEvent( QDropEvent *drop){
  QStringList strliste;

	QUriDrag::decodeToUnicodeUris( drop, strliste);

  openFilesDropPaste(strliste);

}

/** Receive a message when the user try to
	* drag something. It extract the selected files,
	* and then transmit their path to the target
	* application
 	*/
void KarchiveurApp::slotDoDragAndDrop(){
	QListViewItem* vi;
	QStrList urlsatransmettre;
	QString fichiersaextraire;
	QString repulterieur;
	QString tmp=NULL;
  int k;


  if(operation==NOARCHIVE)
  	return;

 	//We will extract the archive to a temp folder, so let's clean it
 	process->clearArguments();
 	*process << "rm" << "-fR" <<reptmp;
 	process->start(KProcess::Block);
 	process->clearArguments();
 	*process << "mkdir" << reptmp;
 	process->start(KProcess::Block);

	//Extract only the selected files
	archiveobj->extractArchive(reptmp, 1,tmp);

 	vi=view->getWidgetListe()->firstChild();
 	for(k=0;k<view->getWidgetListe()->childCount();k++)
 	{
 		if((vi->isSelected())&&(strcmp(vi->text(0),"..")!=0)&&repulterieur!=vi->text(6))
 		{
 			repulterieur=vi->text(6);
 			fichiersaextraire+=vi->text(6);
			if(vi->text(6).isEmpty()||vi->text(6).isNull()||vi->text(6)==" ")
	 			fichiersaextraire+=vi->text(0);
 			fichiersaextraire=fichiersaextraire.right(fichiersaextraire.length()-1);
 			fichiersaextraire=reptmp+fichiersaextraire;
			kdDebug()<<QString("*Drag of:%1*").arg(fichiersaextraire)<<endl;
 			urlsatransmettre.append(fichiersaextraire);
 			fichiersaextraire="";
 		}
 		vi=vi->nextSibling();
 	}
	QDragObject *d = new QUriDrag( urlsatransmettre, this );
	d->dragCopy();
}

/** The contextual menu has been activated */
void KarchiveurApp::slotContextualMenuActivated(int i){
	switch(contextualMenu->indexOf(i))
	{
		case 0:
			slotExtractArchive();
			break;
		case 2:
			slotCopy();
			break;
		case 3:
			slotPaste();
			break;
		case 4:
			slotRemoveToTrashSelectedFiles();
			break;
		case 5:
			slotRemoveSelectedFiles();
			break;
		case 7:
			slotDisplayFile();
			break;
		case 8:
			slotDisplayFileProprieties();
			break;
	}
}

/* The user right-clicked on a file in the listview
	* so we'll display a contextual menu */
void KarchiveurApp::slotShowContextualMenu ( QListViewItem *l, const QPoint &pos, int ){
	if(operation==NOARCHIVE)
		return;

	view->getWidgetListe()->setSelected(l, true);
////	contextualMenu->exec(pos);
 QPopupMenu *pop = (QPopupMenu *)factory()->container("popup_list", this);
 pop->popup(pos);
}

/** Display in a dialog box some informations
concerning the selected file */
void KarchiveurApp::slotDisplayFileProprieties(){
	CProprietes propr;
	QListViewItem* l;
	int kindofCompressor;

	l=view->getWidgetListe()->currentItem();
	kindofCompressor=fichier->getKindOfCompressor();

	propr.setNomFichier(l->text(0));
	propr.setCheminFichier(l->text(6));
	propr.setTaille(l->text(1));
	if((kindofCompressor==TAR)||(kindofCompressor==TARGZ)||(kindofCompressor==TARBZ2))
		propr.setPermission(l->text(5));

	propr.exec();
}

/** The mouse button was clicked on the listview */
void KarchiveurApp::slotMouseClick(QListViewItem*){
	if(operation==READING)
	{
		if(KMessageBox::questionYesNo(this, i18n("Stop opening of current archive ?"), i18n("KArchiver")) == KMessageBox::Yes)
			archiveobj->stopReading();
	}
	else 	if(operation==EXTRACTING)
	{
		if(KMessageBox::questionYesNo(this, i18n("Stop extracting current archive ?"), i18n("KArchiver")) == KMessageBox::Yes)
			archiveobj->stopExtracting();
	}
}

/** The user selected paste */
void KarchiveurApp::slotPaste(){
	QStringList urls;

	if ( QUriDrag::decodeToUnicodeUris(QApplication::clipboard()->data(), urls) )
		openFilesDropPaste(urls);
}

/** Some files were dropped or pasted; they have
been decoded in strliste, so just open them ! */
void KarchiveurApp::openFilesDropPaste( QStringList strliste ){
  QStrList strlisteSansPath;	//Contains the names WITHOUT their path of dropped files
  QStrList strlisteAvecPath;	//Contains the names WITH their path of dropped files
  QString str;
  QString chemin;
  CAddFiles dlgTypeAction;
  QString archivename;
  int kindofCompressor;
  
  strlisteSansPath.clear();
  strlisteAvecPath.clear();
  
  for ( QStringList::Iterator it=strliste.begin(); it!=strliste.end(); ++it)
    {
      kdDebug()<<QString("Drop:%1*").arg(*it)<<endl;
      str=openRemoteFile(KURL((*it)/*.latin1()*/));
      strlisteAvecPath.append(str);
      //Let's keep only the filename
      str=str.right(str.length()-str.findRev('/',str.length()-2)-1);
      if(str.right(1)=="/")
	str=str.remove(str.length()-1,1);
      strlisteSansPath.append(str);
      chemin=(*it)/*.latin1()*/;
    }
  
  //The path, useful for the QStrList without path
  chemin=chemin.left(chemin.findRev('/',chemin.length()-2));
  chemin=chemin.right(chemin.length()-chemin.find(':')-1);
  fichier->setCheminAcces(chemin);
  navigateur->setChemin(fichier->getCheminAcces());
  kdDebug()<<QString("path:%1 *operation*%2 *").arg(chemin).arg(operation)<<endl;
  
  //We'll check if we have dragged only one file: a valid archive
  archivename=fichier->getNomFichier(); //Backup current archive name
  
  if(operation==NOARCHIVE)
    dlgTypeAction.canAddToCurrentArchive(false);
  
  if(strlisteAvecPath.count()==1)
    {
      //We have dropped one file and it is an archive, so modify the dialog box
      kdDebug()<<("Archive dropped\n");
      fichier->setNomArchive(strlisteAvecPath.first());
      if(fichier->findKindOfCompressor()==0)
	{
	  dlgTypeAction.setCheckBoxArchive();
	  operation=STANDBY;
	  displayArchiveContent();
	  return;
	}
    }
  
  if(!dlgTypeAction.exec())
    {
      led->setColor(QColor("orange"));
      operation=STANDBY;
      return;
    }
  switch(dlgTypeAction.getAddFilesChoice())
    {
    case 0:
      //Add to this archive
      fichier->setNomArchive(archivename);
      fichier->findKindOfCompressor();
      operation=STANDBY;
      break;
    case 1:
      //Add to another archive
      fichier->setNomArchive(dlgTypeAction.getOtherArchiveName());
      fichier->findKindOfCompressor();
      kdDebug()<<QString("Add to archive:%1").arg(dlgTypeAction.getOtherArchiveName())<<endl;
      if(archiveobj!=NULL)
	delete archiveobj;
      
      switch(fichier->getKindOfCompressor())
    	{
	case TARGZ:
	  archiveobj=new CTarGz();
	  break;
	case TARBZ2:
	  archiveobj=new CTarBz2();
	  break;
	case TAR:
	  archiveobj=new CTar();
	  break;
	case ZIP:
	  archiveobj=new CZip();
	  break;
	case ARJ:
	  archiveobj=new CArj();
	  break;
	case LHA:
	  archiveobj=new CLha();
	  break;
	case RAR:
	  archiveobj=new CRar();
	  break;
	case GZIP:
	  archiveobj=new CGz();
	  break;
	case BZIP2:
	  archiveobj=new CBz2();
	  break;
	default:
	  led->setColor(QColor("orange"));
	  operation=NOARCHIVE;
	  return;
	  break;
    	}
      connect(archiveobj, SIGNAL(archiveReadEnded()), this, SLOT(slotEndProcess()));
      archiveobj->setReadArchiveWithStream(lecturearchiveflot);
      archiveobj->setArchiveName(fichier->getNomFichier());
      operation=STANDBY;
      break;
    case 2:
      //Create a new archive
      fichier->setNomArchive(dlgTypeAction.getNewArchiveName());
      fichier->findKindOfCompressor();
      kdDebug()<<QString("Add to this new archive:%1").arg(dlgTypeAction.getNewArchiveName())<<endl;
      operation=NOARCHIVE;
      break;
    case 3:
      //Open current archive
      operation=STANDBY;
      displayArchiveContent();
      return;
      break;
    case -1:
      led->setColor(QColor("orange"));
      return;
      break;
    }
  
  archivename=fichier->getNomFichier();
  
  if(operation!=NOARCHIVE)
    {
      //An archive is already loaded
      
      kindofCompressor=fichier->getKindOfCompressor();
      
      if((kindofCompressor==TAR)||(kindofCompressor==TARGZ)||(kindofCompressor==TARBZ2))
	{
	  operation=ADDING;
	  if(incluretoutchemin)
	    addToArchive(archivename,strlisteAvecPath,FALSE,0,chemin);
	  else
	    addToArchive(archivename,strlisteSansPath,FALSE,0,chemin);
	}
      else if((kindofCompressor==ZIP)||(kindofCompressor==LHA)||(kindofCompressor==RAR))
	{
	  operation=ADDING;
	  addToArchive(archivename,strlisteAvecPath,FALSE,0,chemin);
	}
      else
	{
	  KMessageBox::error( this, i18n("Files can be added only in\n.tar, .tar.gz, .tar.bz2 .zip .rar .lha"), "karchiver" );
	  led->setColor(QColor("orange"));
	  return;
	}
    }
  else
    {
      //We will create an archive
      led->setColor(QColor("red"));
      
      if(archivename.isEmpty())
	{
	  slotStatusMsg(i18n("Archive not created"));
	  led->setColor(QColor("orange"));
	  return;
	}
      fichier->setNomArchive(archivename);
      navigateur->setChemin(fichier->getCheminAcces());
      if(fichier->findKindOfCompressor()==1)
	{
	  slotStatusMsg(i18n("Unknown archive type"));
	  led->setColor(QColor("orange"));
	  return;
	}
      kindofCompressor=fichier->getKindOfCompressor();
      ///		operation=AJOUTCREATION;
      if(strcmp(fichier->getProgrCompresseur(),"tar")==0)
	{
	  switch(kindofCompressor)
	    {
	    case TARGZ:
	      archiveobj=new CTarGz();
	      break;
	    case TARBZ2:
	      archiveobj=new CTarBz2();
	      break;
	    case TAR:
	      archiveobj=new CTar();
	      break;
	    }
	  kdDebug()<<QString("Will create Tar based with rel.path:%1*").arg(chemin)<<endl;
	  if(incluretoutchemin)
	    archiveobj->createArchive(archivename,strlisteAvecPath,chemin);
	  else
	    archiveobj->createArchive(archivename,strlisteSansPath,chemin);
	  operation=ADDCREATE;
	  displayArchiveContent();
	}
      else if(strcmp(fichier->getProgrCompresseur(),"unzip")==0)
	{
	  archiveobj=new CZip();
	  //			archiveobj->createArchive(archivename,strliste);
	  kdDebug()<<QString("Will create Zip based with rel.path:%1*").arg(chemin)<<endl;
	  if(incluretoutchemin)
	    archiveobj->createArchive(archivename,strlisteAvecPath,chemin);
	  else
	    archiveobj->createArchive(archivename,strlisteSansPath,chemin);
	  displayArchiveContent();
	}
      else if((strcmp(fichier->getProgrCompresseur(),"gzip")==0)&&(strlisteAvecPath.count()==1))
	{
	  fichierIO=fopen(archivename,"w");
	  kindofCompressor=GZIP;
	  process->clearArguments();
	  *process << "gzip" << "-c" << str;
	  process->start(KProcess::NotifyOnExit, KProcess::Stdout);
	}
      else if((strcmp(fichier->getProgrCompresseur(),"bzip2")==0)&&(strliste.count()==1))
	{
	  fichierIO=fopen(archivename,"w");
	  kindofCompressor=BZIP2;
	  process->clearArguments();
	  *process << "bzip2" << "-c" << str;
	  process->start(KProcess::NotifyOnExit, KProcess::Stdout);
	}
      else
	{
	  KMessageBox::error( this,i18n("Files can be added only in\n .tar, .tar.gz, .tar.bz2"), "karchiveur" );
	  led->setColor(QColor("orange"));
	  return;
	}
      led->setColor(KLed::green);
    }
}

/** The user try to copy a karchiveur's file to
the clipboard */
void KarchiveurApp::slotCopy(){
	QListViewItem* vi;
	QStrList urlsatransmettre;
	QString fichiersaextraire;
	QString repulterieur;
	QString tmp=NULL;
  int k;

  if(operation==NOARCHIVE)
  	return;

 	//We will extract the archive to a temp folder, so let's clean it
 	process->clearArguments();
 	*process << "rm" << "-fR" <<reptmp;
 	process->start(KProcess::Block);
 	process->clearArguments();
 	*process << "mkdir" << reptmp;
 	process->start(KProcess::Block);

	//Extract only the selected files
	archiveobj->extractArchive(reptmp, 1,tmp);

 	vi=view->getWidgetListe()->firstChild();
 	for(k=0;k<view->getWidgetListe()->childCount();k++)
 	{
 		if((vi->isSelected())&&(strcmp(vi->text(0),"..")!=0)&&repulterieur!=vi->text(6))
 		{
 			repulterieur=vi->text(6);
 			fichiersaextraire+=vi->text(6);
			if(vi->text(6).isEmpty()||vi->text(6).isNull()||vi->text(6)==" ")
	 			fichiersaextraire+=vi->text(0);
 			fichiersaextraire=fichiersaextraire.right(fichiersaextraire.length()-1);
 			fichiersaextraire=reptmp+fichiersaextraire;
//	 			fichiersaextraire+=vi->text(0);
 			urlsatransmettre.append(fichiersaextraire);
			kdDebug()<<QString("In copy:addURL:%1*").arg(fichiersaextraire)<<endl;
 			fichiersaextraire="";
 		}
 		vi=vi->nextSibling();
 	}

	kapp->clipboard()->setData( new QUriDrag( urlsatransmettre, this ));

}

/** extract the selected files to trash and then
remove them from the archive */
void KarchiveurApp::slotRemoveToTrashSelectedFiles(){
  slotStatusMsg(i18n("Move all selected files to trash..."));

	if(operation!=NOARCHIVE)
	{
		int i;
		int k;
		QListViewItem *li=view->getWidgetListe()->firstChild();
		QString txt;
		QString repTrash;
		KConfig conf("kdeglobals"); //To get the "trash" directory

		if(li==0)
		{
		  slotStatusMsg(i18n("First select a file"));
		  led->setColor(QColor("orange"));
			return;
		}

		led->setColor(QColor("red"));
		repTrash=QDir::homeDirPath();
		repTrash+="/Desktop/Trash";
		conf.setGroup("Paths");
		repTrash=conf.readEntry("Trash",repTrash);
		k=view->getWidgetListe()->childCount();

		for(i=0;i<k;i++)
		{
			if(view->getWidgetListe()->isSelected(li))
			{
				txt=li->text(6);
				txt=txt.remove(0,1);
				txt+=li->text(0);
				archiveobj->extractArchive(repTrash, 9,txt);
			}
			if(i!=k)
				li=li->nextSibling();
		}
	}

	slotRemoveSelectedFiles();

	led->setColor(QColor("green"));
  slotStatusMsg(IDS_STATUS_DEFAULT);
}

/** the user has "cutted" some files in the current
archive */
void KarchiveurApp::slotCutt(){
	QListViewItem* vi;
	QStrList urlsatransmettre;
	QString fichiersaextraire;
	QString repulterieur;
	QString tmp=NULL;
  int k;

  kdDebug()<<("IN:slotCutt\n");
  if(operation==NOARCHIVE)
  	return;

 	//We will extract the archive to a temp folder, so let's clean it
 	process->clearArguments();
 	*process << "rm" << "-fR" <<reptmp;
 	process->start(KProcess::Block);
 	process->clearArguments();
 	*process << "mkdir" << reptmp;
 	process->start(KProcess::Block);

	//Extract only the selected files
	archiveobj->extractArchive(reptmp, 1,tmp);

 	vi=view->getWidgetListe()->firstChild();
 	for(k=0;k<view->getWidgetListe()->childCount();k++)
 	{
 		if((vi->isSelected())&&(strcmp(vi->text(0),"..")!=0)&&repulterieur!=vi->text(6))
 		{
 			repulterieur=vi->text(6);
 			fichiersaextraire+=vi->text(6);
			if(vi->text(6).isEmpty()||vi->text(6).isNull()||vi->text(6)==" ")
	 			fichiersaextraire+=vi->text(0);
 			fichiersaextraire=fichiersaextraire.right(fichiersaextraire.length()-1);
 			fichiersaextraire=reptmp+fichiersaextraire;
//	 			fichiersaextraire+=vi->text(0);
 			urlsatransmettre.append(fichiersaextraire);
			kdDebug()<<QString("In cutt:addURL:%1*").arg(fichiersaextraire)<<endl;
 			fichiersaextraire="";
 		}
 		vi=vi->nextSibling();
 	}
	QDragObject *d = new QUriDrag( urlsatransmettre, this );

	kapp->clipboard()->setData( d);

	KMessageBox::information(this, i18n("Paste those files in any application NOW !"), "karchiveur");
	slotRemoveSelectedFiles();

}

/** select all the files in the current archive */
void KarchiveurApp::slotSelectAll(){
	view->getWidgetListe()->selectAll(true);
}

/** unselect all files in the current archive */
void KarchiveurApp::slotUnselectAll(){
	view->getWidgetListe()->selectAll(false);
}

/** find in this archive all items matching the
name name */
void KarchiveurApp::slotFindInThisArchive( QString name ){
	QListViewItem *vi;
	int k;

	kdDebug()<<QString("Find:%1*%2*").arg(name).arg(indexfind)<<endl;
 	vi=view->getWidgetListe()->firstChild();
 	for(k=0;k<view->getWidgetListe()->childCount();k++)
 	{
 		if((k>indexfind)&&(vi->text(0).contains(name)!=0))
 		{
			view->getWidgetListe()->setCurrentItem(vi);
			vi->setSelected(true);
			indexfind=k;
			return;
 		}
 		vi=vi->nextSibling();
 	}

 	KMessageBox::information(this, i18n("No more occurrences found!"), "karchiveur");
 	indexfind=-1;
}
/** the user closed the find in this archive tool */
void KarchiveurApp::slotCloseFind(){
	find->hide();
	indexfind=-1;
}

/** Use the find in this archive tool */
void KarchiveurApp::slotUseFind(){
	find->show();
	indexfind=-1;
}

/** Remove form current archive the selected fies,
with an undo option (i.e. extract to reptmpundo, then
remove) */
void KarchiveurApp::slotRemoveWithUndoSelectedFiles(){
  slotStatusMsg(i18n("Deleting selected files..."));

	if(operation!=NOARCHIVE)
	{
		int i;
		int k;
		QListViewItem *li=view->getWidgetListe()->firstChild();
		QString txt;
		QString repTrash;

		if(li==0)
		{
		  slotStatusMsg(i18n("First select a file"));
		  led->setColor(QColor("orange"));
			return;
		}

		led->setColor(QColor("red"));

		k=view->getWidgetListe()->childCount();
		listUndo.clear();
		listRedo.clear();

		for(i=0;i<k;i++)
		{
			if(view->getWidgetListe()->isSelected(li))
			{
				txt=li->text(6);
				txt=txt.remove(0,1);
				txt+=li->text(0);
				listUndo.append(txt);
				listRedo.append(txt);
				archiveobj->extractArchive(reptmpundo, 9, txt);
			}
			if(i!=k)
				li=li->nextSibling();
		}
	}

	slotRemoveSelectedFiles();

	led->setColor(QColor("green"));
  slotStatusMsg(IDS_STATUS_DEFAULT);
}

/** Undo the previous operation, i.e. the deletion
od files from archive */
void KarchiveurApp::slotUndo(){
	if(listUndo.isEmpty())
	{
		led->setColor(QColor("orange"));
		slotStatusMsg(i18n("Nothing to undo !"));
		return;
	}

	addToArchive(fichier->getNomFichier(), listUndo, true, 0, reptmpundo);
	listUndo.clear();
}

/** Redo the last operation (i.e. remove once
again the selected file) */
void KarchiveurApp::slotRedo(){
	if((listRedo.isEmpty())||(!listUndo.isEmpty()))
	{
		led->setColor(QColor("orange"));
		slotStatusMsg(i18n("Nothing to redo !"));
		return;
	}

	slotRemoveWithUndoThoseFiles(listRedo);
	listRedo.clear();
}

/** Remove from current archives the files in
files, with undo possibility */
void KarchiveurApp::slotRemoveWithUndoThoseFiles( QStrList files ){
  slotStatusMsg(i18n("Deleting selected files..."));

	if(operation!=NOARCHIVE)
	{
		QString txt;
		QString repTrash;

		led->setColor(QColor("red"));

		listUndo.clear();
		listRedo.clear();

		for(txt=files.first(); txt; txt=files.next())
		{
				listUndo.append(/*reptmpundo+"/"+*/txt);
				listRedo.append(txt);
				archiveobj->extractArchive(reptmpundo, 9,txt);
		}
	}

	slotRemoveThoseFiles(files);

	led->setColor(QColor("green"));
  slotStatusMsg(IDS_STATUS_DEFAULT);
}

/** Remove the files in files from current archive */
void KarchiveurApp::slotRemoveThoseFiles( QStrList files ){
	led->setColor(QColor("red"));
  slotStatusMsg(i18n("Deleting selected files..."));

	if(operation!=NOARCHIVE)
	{
		QString txt;
		QStrList strlist;

		operation=STANDBY;
		archiveobj->removeFilesFromArchive(files);
		operation=STANDBY;
	}

	displayArchiveContent();
	led->setColor(QColor("green"));
  displayMessageArchiveSize();
}

/** a process in CArchive has ended. So make
the necessary with the operation variable */
void KarchiveurApp::slotEndProcess(){
	int kindofCompressor=fichier->getKindOfCompressor();
	if(operation==SCRIPT)
	{
		avancement->setValue(0);

		QString nomfichselect;
		QString repbase;
		QString repbasetmp;
		CScript objetscript;
		QListViewItem *li;
		int typescript;

		operation=STANDBY;
		repbase="";
		li=view->getWidgetListe()->currentItem();
		if(li==NULL)
		{
			li=view->getWidgetListe()->firstChild();
			while(li)
			{
				if(strcmp(li->text(0),"..")!=0)
					break;
				li=li->nextSibling();
			}
		}
		nomfichselect=li->text(0);
		if(nomfichselect=="..")
		{
			led->setColor(QColor("orange"));
			return;
		}
		if((kindofCompressor==TAR)||(kindofCompressor==TARGZ)||(kindofCompressor==TARBZ2)||(kindofCompressor==ZIP))
		{
			repbasetmp=li->text(6);
			repbasetmp.remove(0,1);
			nomfichselect=repbasetmp+nomfichselect;
			li=view->getWidgetListe()->firstChild();
			repbase=li->text(6);
			while(li)
			{
				repbasetmp=li->text(6);
				if(repbase.length()>repbasetmp.length())
					repbase=repbasetmp;
				li=li->nextSibling();
			}
			repbase.remove(0,1);
		}

		repbase=reptmp+repbase;
		kdDebug()<<QString("StartingWizard: base path %1").arg(repbase)<<endl;

		typescript=objetscript.demarrerAssistant(nomfichselect,repbase,fichier->getNomFichier());
		if(typescript==2)
		{
			//User has choosed to convert the archive
			doArchiveConversion(objetscript.getTypeConversion(),objetscript.getRepertoireConversion());
		}
		else if(typescript==3)
		{
			//User has choosed to split/unsplit archive
			if(objetscript.getRepertoireCutt()!=NULL)
				doCuttUnCutt(objetscript.getRepertoireCutt());
		}
		led->setColor(QColor("green"));
	}
	else if(operation==CONVERSION)
	{
	  kdDebug()<<("\nEntering Conversion\n");
		int type=-1;
		QString baserep;
		QString newarchivename;
		QString oldarchivename;

		KFileDialog boiteSauver(extraction->getCheminAcces(),"*",0,i18n("Choose new archive name"),false);
		QString programme=fichier->getNomFichier();
		boiteSauver.setName(i18n("Convert archive"));
		newarchivename=boiteSauver.getSaveFileName();

		if(newarchivename==NULL)
		{
			operation=STANDBY;
			led->setColor(QColor("orange"));
			return;
		}
		repaint();
		
		oldarchivename=fichier->getNomFichier();
		fichier->setNomArchive(newarchivename);
		if(fichier->findKindOfCompressor()==1)
		{
			slotStatusMsg(i18n("Unknown archive type"));
			led->setColor(QColor("orange"));
			fichier->setNomArchive(oldarchivename);
			fichier->findKindOfCompressor();
			return;
		}
		operation=NOARCHIVE;
		kindofCompressor=fichier->getKindOfCompressor();
		
		switch(kindofCompressor)
		{
			case TAR:
				type=0;
				break;
			case TARGZ:
				type=1;
				break;
			case TARBZ2:
				type=2;
				break;
			case ZIP:
				type=3;
				break;
			case GZIP:
				type=4;
				break;
			case BZIP2:
				type=5;
			case RAR:
				type=6;
				break;
			case LHA:
				type=7;
				break;
			default:
				type=-1;
				break;
		}
		baserep=newarchivename.left(newarchivename.findRev('/'));
//		fichier->setNomArchive(oldarchivename); //It'll be set to the new name in doArchiveConversion
//		fichier->findKindOfCompressor();

		kdDebug()<<QString("Conversion of %1*\n").arg(baserep).arg(type);
		doArchiveConversion(type,baserep);
	return;
	}
	operation=STANDBY;
}

/** Initialize the actions and the GUI */
void KarchiveurApp::initActions(){
  fileNewWindow=new KAction( i18n("New &Window"), kapp->miniIcon(), 0, this, SLOT(slotFileNewWindow()), actionCollection(),"file_new_window");
  fileNewArchive=new KAction( i18n("&New"), "filenew", 0, this, SLOT(slotFileNew()), actionCollection(),"file_new_archive" );
  fileOpenArchive=new KAction( i18n("&Open..."), "fileopen", 0, this, SLOT(slotFileOpen()), actionCollection(),"file_open_archive" );
  fileSearchArchive=new KAction( i18n("Search for archives"), "find", 0, this, SLOT(slotFileSearch()), actionCollection(),"file_search_archive" );
  fileOpenRecent = KStdAction::openRecent( this, SLOT(slotFileOpenRecent(const KURL&)), actionCollection(), "file_open_recent_kar");
  fileConvertArchive=new KAction( i18n("Convert &as"), "filesave", 0, this, SLOT(slotFileSaveAs()), actionCollection(),"file_convert_archive" );
  fileInfosArchive=new KAction( i18n("View archive infos"), "documentinfo", 0, this, SLOT( slotDisplayArchiveInfos()), actionCollection(),"file_infos_archive" );
  filePrint = KStdAction::print( this, SLOT(slotFilePrint()), actionCollection(), "file_print_kar");
  fileCloseArchive=new KAction( i18n("C&lose Window"), "fileclose", 0, this, SLOT( slotFileClose()), actionCollection(),"file_close_archive" );
  fileQuit = KStdAction::quit(this, SLOT(slotFileQuit()), actionCollection());

  ///////////////////////////////////////////////////////////////////
  // menuBar entry archiveMenu
  archiveExtract=new KAction(i18n("Extract To"), "extract", 0, this, SLOT( slotExtractArchive()), actionCollection(),"archive_extract" );
  archiveAdd=new KAction(i18n("Add files"), "compress", 0, this, SLOT( slotAddFiles()), actionCollection(),"archive_add" );
  archiveView=new KAction(i18n("View file"), "exec", 0, this, SLOT( slotDisplayFile()), actionCollection(),"archive_view" );
  archiveViewSelection=new KAction(i18n("View selected"), "view", 0, this, SLOT( slotDisplaySelectedFiles()), actionCollection(),"archive_view_selection" );
  archiveDeleteSelection=new KAction(i18n("Delete Files"), "editcut", 0, this, SLOT( slotRemoveWithUndoSelectedFiles()), actionCollection(),"archive_delete_selection" );
  archiveDeleteToTrashSelection=new KAction(i18n("Move to trash"), "edittrash", 0, this, SLOT( slotRemoveToTrashSelectedFiles()), actionCollection(),"archive_delete_to_trash_selection" );
  archiveWizard=new KAction(i18n("Launch the wizard"), "wizard", 0, this, SLOT( slotWizard()), actionCollection(),"archive_wizard" );
  archiveProprieties=new KAction(i18n("Proprieties"), 0, this, SLOT( slotDisplayFileProprieties()), actionCollection(),"archive_proprieties" );

  ///////////////////////////////////////////////////////////////////
  // menuBar entry editeMenu
  editUndo=KStdAction::undo( this, SLOT( slotUndo()), actionCollection(),"edit_undo_kar" );
  editRedo=KStdAction::redo( this, SLOT( slotRedo()), actionCollection(),"edit_redo_kar" );
  editCut=KStdAction::cut( this, SLOT( slotCutt()), actionCollection(),"edit_cut_kar" );
  editCopy=KStdAction::copy( this, SLOT( slotCopy()), actionCollection(),"edit_copy_kar" );
  editPaste=KStdAction::paste( this, SLOT( slotPaste()), actionCollection(),"edit_paste_kar" );
  editSelectAll=KStdAction::selectAll( this, SLOT( slotSelectAll()), actionCollection(),"edit_select_all_kar" );
  editUnselectAll=new KAction(i18n("Select none"), 0, this, SLOT( slotUnselectAll()), actionCollection(),"edit_unselect_all" );
  editFind=new KAction(i18n("Find..."), "find", 0, this, SLOT( slotUseFind()), actionCollection(),"edit_find_kar" );

  ///////////////////////////////////////////////////////////////////
  // menuBar entry diskMenu
	diskSplit=new KAction(i18n("Split/Unsplit"), 0, this, SLOT(slotDiskCutUncut()), actionCollection(), "disk_split");

  ///////////////////////////////////////////////////////////////////
  // menuBar entry viewMenu
  viewToolbar=new KToggleAction(i18n("&Toolbar"), 0, this, SLOT(slotViewToolBar()), actionCollection(), "view_toolbar" );
  viewStatusbar=new KToggleAction(i18n("&Statusbar"), 0, this, SLOT(slotViewStatusBar()), actionCollection(), "view_statusbar" );
  viewArchiveBrowser=new KToggleAction(i18n("&File browser"), 0, this, SLOT(slotViewButineur()), actionCollection(), "view_archive_browser" );
  viewPreferences=new KAction(i18n("preferences"), "configure", 0, this, SLOT(slotSetOptions()), actionCollection(), "view_preferences" );
  KStdAction::configureToolbars(this, SLOT(slotEditToolbars()), actionCollection(), "view_edit_toolbar" );

  fileNewWindow->setStatusText(i18n("Opens a new application window"));
  fileNewArchive->setStatusText(i18n("Creates a new archive"));
  fileOpenArchive->setStatusText(i18n("Creates a new archive"));
  fileSearchArchive->setStatusText(i18n("Search for existing archive"));
  fileConvertArchive->setStatusText(i18n("Convert current archive format to..."));
  archiveDeleteSelection->setStatusText(i18n("Delete selected files"));
  archiveExtract->setStatusText(i18n("Extract to..."));
  archiveAdd->setStatusText(i18n("Add files to this archive"));
  archiveView->setStatusText(i18n("View the current file"));
  archiveViewSelection->setStatusText(i18n("View all selected files"));
  fileInfosArchive->setStatusText(i18n("Display info on current archive"));
  archiveWizard->setStatusText(i18n("Launch a wizard to configure & compile this archive..."));
  diskSplit->setStatusText(i18n("Split archive in 1.4Mb blocks"));
  viewArchiveBrowser->setStatusText(i18n("Enables / disables the file navigator"));
  viewPreferences->setStatusText(i18n("Modify options of KArchiver"));
  editCut->setStatusText(i18n("move the files that were copied to the clipboard"));
  editCopy->setStatusText(i18n("Extract selected files and copy them to the clipboard"));
  editPaste->setStatusText(i18n("Add the files that were copied to the clipboard"));
  editFind->setStatusText(i18n("Find a filename in this archive"));
  editUndo->setStatusText(i18n("Undo the last operation"));
  editRedo->setStatusText(i18n("Redo the last operation"));

  createGUI();
}

void KarchiveurApp::slotEditToolbars()
{
  KEditToolbar dlg(actionCollection());
  if (dlg.exec())
    createGUI();
}

/** Extract the archives @archives to the directory @extractdir.
	If @promptdir is true, we'll show the extract dialog, to let the user
	choosing its extract dir */
void KarchiveurApp::extractFromCommandLine( QStrList archives, bool promptdir, QCString extractdir, bool chooseExtractDirFromArchive ){
	QStrList strlisteSansPath;	//Contains the names WITHOUT their path of dropped files
	QStrList strlisteAvecPath;	//Contains the names WITH their path of dropped files
	QString str;
	QString chemin;
	QString archivename;
	QString message;
	QString nom;
	QString tmp=NULL;
	int kindofCompressor;
	QStringList strliste=QStringList::fromStrList(archives);
	
  strlisteSansPath.clear();
  strlisteAvecPath.clear();

  for ( QStringList::Iterator it=strliste.begin(); it!=strliste.end(); ++it)
    {
      kdDebug()<<QString("ExtractFromCmdLine:%1*").arg(*it)<<endl;
      str=openRemoteFile(KURL((*it)/*.latin1()*/));
      strlisteAvecPath.append(str);
      //Let's keep only the filename
      str=str.right(str.length()-str.findRev('/',str.length()-2)-1);
      if(str.right(1)=="/")
			str=str.remove(str.length()-1,1);
      strlisteSansPath.append(str);
      chemin=(*it)/*.latin1()*/;
    }

  //The path, useful for the QStrList without path
  chemin=chemin.left(chemin.findRev('/',chemin.length()-2));
  chemin=chemin.right(chemin.length()-chemin.find(':')-1);
  fichier->setCheminAcces(chemin);
  navigateur->setChemin(fichier->getCheminAcces());
  kdDebug()<<QString("path:%1 *operation*%2 *").arg(chemin).arg(operation)<<endl;

  //We'll check if we have dragged only one file: a valid archive
  archivename=fichier->getNomFichier(); //Backup current archive name

  if(strlisteAvecPath.count()!=1)
  	return;
      //We have dropped one file and it is an archive, so modify the dialog box
      fichier->setNomArchive(strlisteAvecPath.first());
	if(fichier->findKindOfCompressor()==1)
	{
		slotStatusMsg(i18n("Unknown archive type"));
		operation=NOARCHIVE;
		led->setColor(QColor("orange"));
		return;
	}

	  operation=STANDBY;

	led->setColor(QColor("red"));


	nom=fichier->getNomFichier();
	setCaption(nom);

  fileOpenRecent->addURL(KURL(fichier->getNomFichier()));
	navigateur->setChemin(fichier->getCurrentArchiveDirectory());

	view->getWidgetListe()->clear();
	for(int i=0;i<7;i++)
		view->getWidgetListe()->setColumnWidth(i,50);
	operation=READING;

	kindofCompressor=fichier->getKindOfCompressor();

	if(archiveobj!=NULL)
		delete archiveobj;
kdDebug()<<QString("ExtrCmdLine:Kind of compressor: %1\n").arg(kindofCompressor);
	switch(kindofCompressor)
	{
		case TARGZ:
			archiveobj=new CTarGz();
			break;
		case TARBZ2:
			archiveobj=new CTarBz2();
			break;
		case TAR:
			archiveobj=new CTar();
			break;
		case ZIP:
			archiveobj=new CZip();
			break;
		case ARJ:
			archiveobj=new CArj();
			break;
		case LHA:
			archiveobj=new CLha();
			break;
		case RAR:
			archiveobj=new CRar();
			break;
		case GZIP:
			archiveobj=new CGz();
			break;
		case BZIP2:
			archiveobj=new CBz2();
			break;
	}
	connect(archiveobj, SIGNAL(archiveReadEnded()), this, SLOT(slotFileQuit()));
	archiveobj->setReadArchiveWithStream(lecturearchiveflot);
	archiveobj->setArchiveName(fichier->getNomFichier());
//	archiveobj->displayArchiveContent();
  extraction->afficheChemin();
  operation=STANDBY; //not set to EXTRACTING as we should call  slotEndProcess. However, heren slotFileQuit seems to be called first
   if(promptdir)
   {
  		if(extraction->exec())
  		{
	      archiveobj->extractArchive(extraction->getCheminAcces(),extraction->toutExtraire(),tmp);
  		}
  		else
  			return;
  	}
  	else
  	{
      QString extr(extractdir);

      if(chooseExtractDirFromArchive)
      		extr=chemin;
      archiveobj->extractArchive(extr,0,tmp);
    }
}

/** Check wether karchiver is lauched for the first time. If so, launch
a wizard to configure the main features */
void KarchiveurApp::checkFirstInstall(){
  	int rep=1;
  	QString arkpath, arkpath2;
	CFirstInstallWizard firstinstallwizard(this, "1st_install", true);

  config->setGroup("First Time Install");
  if( config->readBoolEntry("first run", true)==true )
  	if( firstinstallwizard.exec() )
  {
	  config->setGroup("First Time Install");
	  config->writeEntry("first run", false);
	
		config->setGroup("OPTIONS");
		if(firstinstallwizard.rb_wind_like->isChecked())
			config->writeEntry("SelectionMode",1);
		else
			config->writeEntry("SelectionMode",0);
	
		config->setGroup("LASTDIRS");
		if(firstinstallwizard.rb_extract_lastdir->isChecked())
			rep=0;
		else if(firstinstallwizard.rb_extract_homedir->isChecked())
			rep=1;
		else if(firstinstallwizard.rb_extract_shelldir->isChecked())
			rep=2;
		config->writeEntry("OptionExtraction",rep);
		
		if(firstinstallwizard.rb_open_lastdir->isChecked())
			rep=0;
		else if(firstinstallwizard.rb_open_homedir->isChecked())
			rep=1;
		else if(firstinstallwizard.rb_open_shelldir->isChecked())
			rep=2;
		config->writeEntry("OptionOuverture",1);
		
		if(firstinstallwizard.cb_use_karchiver->isChecked())
		{
			arkpath=KGlobal::dirs()->findResource("data", "konqueror/servicemenus/arkservicemenu.desktop");
			arkpath2=KGlobal::dirs()->findResource("services", "arkpart.desktop");
			kdDebug()<<QString("Will gzip %1 %2\n").arg(arkpath).arg(arkpath2);
			KShellProcess proc;
			proc.clearArguments();
			if(KStandardDirs::findExe("xterm")==NULL)
				proc << "konsole";
			else
				proc << "xterm";
			proc << "-e";
			proc << "su";
			proc << "-c";
			proc << "'";
			proc << "gzip";
			proc  << (const char*)arkpath;
			proc << ";";
			proc << "gzip" << arkpath2;
			proc << "'";
			proc.start(KProcess::Block);
			KMessageBox::information(this, i18n("Has gziped ark files:\n")+arkpath+"\n"+arkpath2, i18n("KArchiver setup"));		
		}
	config->sync();
	readOptions();
	}
}

/** Automaticaly create a compressed file containing @files.
If there is a single file or directory, create an archive
named files.tar.gz. Otherwise, prompt the user for a name */
void KarchiveurApp::createArchiveFromCommandLine( QStrList files ){
  QStrList strlisteSansPath;	//Contains the names WITHOUT their path of dropped files
  QStrList strlisteAvecPath;	//Contains the names WITH their path of dropped files
  QString str;
  QString chemin;
  CAddFiles dlgTypeAction;
  QString archivename;
  int kindofCompressor;
  QStringList strliste;

  strliste=QStringList::fromStrList(files);
  strlisteSansPath.clear();
  strlisteAvecPath.clear();

  for ( QStringList::Iterator it=strliste.begin(); it!=strliste.end(); ++it)
    {
      kdDebug()<<QString("Drop:%1*").arg(*it)<<endl;
      str=openRemoteFile(KURL((*it)/*.latin1()*/));
      strlisteAvecPath.append(str);
      //Let's keep only the filename
      str=str.right(str.length()-str.findRev('/',str.length()-2)-1);
      if(str.right(1)=="/")
	str=str.remove(str.length()-1,1);
      strlisteSansPath.append(str);
      chemin=(*it)/*.latin1()*/;
    }

  //The path, useful for the QStrList without path
  chemin=chemin.left(chemin.findRev('/',chemin.length()-2));
  chemin=chemin.right(chemin.length()-chemin.find(':')-1);
  fichier->setCheminAcces(chemin);
  navigateur->setChemin(fichier->getCheminAcces());
  kdDebug()<<QString("path:%1 *operation*%2 *").arg(chemin).arg(operation)<<endl;

  //We'll check if we have dragged only one file: a valid archive
  archivename=fichier->getNomFichier(); //Backup current archive name

  if(operation==NOARCHIVE)
    dlgTypeAction.canAddToCurrentArchive(false);

  if(strlisteAvecPath.count()==1)
  {
     //We have dropped one file; so create archivename from it.
     kdDebug()<<("Will create an archive from this file\n");
     operation=NOARCHIVE;
	  archivename=strlisteAvecPath.first();
	  archivename+=".tar.gz";
      fichier->setNomArchive( archivename );
      fichier->findKindOfCompressor();
  }
  else
  {
    if(!dlgTypeAction.exec())
    {
        led->setColor(QColor("orange"));
        operation=STANDBY;
        return;
    }
    switch(dlgTypeAction.getAddFilesChoice())
      {
      case 0:
        //Add to this archive
        fichier->setNomArchive(archivename);
        fichier->findKindOfCompressor();
        operation=STANDBY;
        break;
      case 1:
        //Add to another archive
        fichier->setNomArchive(dlgTypeAction.getOtherArchiveName());
        fichier->findKindOfCompressor();
        kdDebug()<<QString("Add to archive:%1").arg(dlgTypeAction.getOtherArchiveName())<<endl;
        if(archiveobj!=NULL)
  			delete archiveobj;

       switch(fichier->getKindOfCompressor())
      	{
         	case TARGZ:
         	  archiveobj=new CTarGz();
         	  break;
         	case TARBZ2:
         	  archiveobj=new CTarBz2();
         	  break;
         	case TAR:
         	  archiveobj=new CTar();
         	  break;
         	case ZIP:
         	  archiveobj=new CZip();
         	  break;
         	case ARJ:
         	  archiveobj=new CArj();
         	  break;
         	case LHA:
         	  archiveobj=new CLha();
         	  break;
         	case RAR:
         	  archiveobj=new CRar();
         	  break;
         	case GZIP:
         	  archiveobj=new CGz();
         	  break;
         	case BZIP2:
         	  archiveobj=new CBz2();
         	  break;
         	default:
         	  led->setColor(QColor("orange"));
         	  operation=NOARCHIVE;
         	  return;
         	  break;
      	}
        connect(archiveobj, SIGNAL(archiveReadEnded()), this, SLOT(slotEndProcess()));
        archiveobj->setReadArchiveWithStream(lecturearchiveflot);
        archiveobj->setArchiveName(fichier->getNomFichier());
        operation=STANDBY;
        break;
      case 2:
        //Create a new archive
        fichier->setNomArchive(dlgTypeAction.getNewArchiveName());
        fichier->findKindOfCompressor();
        kdDebug()<<QString("Add to this new archive:%1").arg(dlgTypeAction.getNewArchiveName())<<endl;
        operation=NOARCHIVE;
        break;
      case 3:
        //Open current archive
        operation=STANDBY;
        displayArchiveContent();
        return;
        break;
      case -1:
        led->setColor(QColor("orange"));
        return;
        break;
      }
 	  archivename=fichier->getNomFichier();
	}
	

   //We will create an archive
   led->setColor(QColor("red"));

   if(archivename.isEmpty())
   {
	   slotStatusMsg(i18n("Archive not created"));
   		led->setColor(QColor("orange"));
	   return;
   }
   fichier->setNomArchive(archivename);
   navigateur->setChemin(fichier->getCheminAcces());
   if(fichier->findKindOfCompressor()==1)
   {
	   slotStatusMsg(i18n("Unknown archive type"));
   		led->setColor(QColor("orange"));
	   return;
   }
   kindofCompressor=fichier->getKindOfCompressor();
   ///		operation=AJOUTCREATION;
   if(strcmp(fichier->getProgrCompresseur(),"tar")==0)
   {
     switch(kindofCompressor)
     {
     case TARGZ:
       archiveobj=new CTarGz();
       break;
     case TARBZ2:
       archiveobj=new CTarBz2();
       break;
     case TAR:
       archiveobj=new CTar();
       break;
     }
     kdDebug()<<QString("Will create Tar based with rel.path:%1*").arg(chemin)<<endl;
     if(incluretoutchemin)
	     archiveobj->createArchive(archivename,strlisteAvecPath,chemin);
     else
   		  archiveobj->createArchive(archivename,strlisteSansPath,chemin);
	   operation=ADDCREATE;
   }
   else if(strcmp(fichier->getProgrCompresseur(),"unzip")==0)
   {
     archiveobj=new CZip();
     kdDebug()<<QString("Will create Zip based with rel.path:%1*").arg(chemin)<<endl;
     if(incluretoutchemin)
	     archiveobj->createArchive(archivename,strlisteAvecPath,chemin);
     else
   		  archiveobj->createArchive(archivename,strlisteSansPath,chemin);
   }
   else
   {
	   KMessageBox::error( this,i18n("Files can be added only in\n .tar, .tar.gz, .tar.bz2"), "karchiveur" );
	   led->setColor(QColor("orange"));
   		return;
   }
   operation=STANDBY;
   //close();
//   delete navigateur;
delete 	archiveobj;
archiveobj=NULL;
exit(0);
   slotFileQuit();
}

#include "karchiveur.moc"
